# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from __future__ import division

import os
import re
import time
import pandas
import logging
import datetime

from django import forms
from functools import wraps
from django.conf import settings


from steelscript.netshark.core.filters import TimeFilter
from steelscript.netshark.core._class_mapping import path_to_class

from steelscript.appfwk.apps.devices.devicemanager import DeviceManager
from steelscript.appfwk.apps.devices.forms import fields_add_device_selection
from steelscript.appfwk.apps.datasource.models import \
    TableField, DatasourceTable, TableQueryBase
from steelscript.appfwk.apps.datasource.forms import \
    fields_add_time_selection, fields_add_resolution
from steelscript.appfwk.libs.fields import Function
from steelscript.wireshark.appfwk.datasources.wireshark_source import \
    fields_add_filterexpr
from steelscript.appfwk.apps.jobs import QueryComplete
from steelscript.appfwk.apps.jobs.models import Job
from steelscript.netshark.appfwk.datasources.netshark import \
    netshark_source_name_choices
from steelscript.netshark.core.filters import BpfFilter

logger = logging.getLogger(__name__)

PCAP_DIR = os.path.join(settings.DATA_CACHE, 'pcaps')


def add_pcap_dir(f):
    return os.path.join(PCAP_DIR, f)


# decorator for bounded methods
# "http://stackoverflow.com/questions/306130
# /python-decorator-makes-function-forget-that-it-belongs-to-a-class"

def logtime(func):
    @wraps(func)
    def inner(*args, **kwargs):
        logger.debug("%s starting" % func.__name__)
        start_time = time.time()
        ret = func(*args, **kwargs)
        end_time = time.time()
        logger.debug("%s finished. It took %s."
                     % (func.__name__,
                        datetime.timedelta(0, end_time - start_time)))
        return ret
    return inner


class PcapDownloadTable(DatasourceTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _query_class = 'PcapDownloadQuery'

    TABLE_OPTIONS = {'aggregated': False,
                     'include_files': False,
                     'include_interfaces': False,
                     'include_persistent': False,
                     'filters': [],
                     'wait_for_data': False,
                     'wait_duration': 10
                     }

    FIELD_OPTIONS = {'resolution': '1s',
                     'resolutions': ('1s', '1m', '15min', '1h')}

    def post_process_table(self, field_options):
        fields_add_device_selection(self, keyword='netshark_device',
                                    label='NetShark', module='netshark',
                                    enabled=True)
        fields_add_time_selection(self, show_start=True,
                                  initial_start_time='now-1m',
                                  show_end=True,
                                  show_duration=False)
        fields_add_resolution(self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'])
        func = Function(netshark_source_name_choices, self.options)
        TableField.create(keyword='netshark_source_name', label='Source',
                          obj=self,
                          field_cls=forms.ChoiceField,
                          parent_keywords=['netshark_device'],
                          dynamic=True,
                          pre_process_func=func)
        fields_add_filterexpr(obj=self)
        self.add_column('filename', datatype='string')


class PcapDownloadQuery(TableQueryBase):

    @logtime
    def download(self, export):
        export.download(self.filename, overwrite=True)

    @property
    def all_pcap_size(self):
        total = 0
        for f in os.listdir(PCAP_DIR):
            if f.endswith('.pcap'):
                try:
                    total += os.path.getsize(add_pcap_dir(f))
                except OSError:
                    logger.warning("Failed to read size of file %s" % f)
        return total

    def delete_oldest_pcap(self):

        oldest_time = 0
        oldest_pcap = None

        for f in os.listdir(PCAP_DIR):
            if f.endswith('.pcap'):
                try:
                    file_time = os.stat(add_pcap_dir(f)).st_mtime
                except OSError:
                    logger.warning("Failed to get time of file %s" % f)
                    continue

                if oldest_time == 0:
                    oldest_time = file_time
                    oldest_pcap = f
                elif file_time < oldest_time:
                    oldest_time = file_time
                    oldest_pcap = f

        if oldest_pcap is None:
            return

        try:
            os.unlink(add_pcap_dir(oldest_pcap))
        except OSError:
            logger.warning("Failed to remove oldest file %s" % oldest_pcap)

        handle = re.sub('\.pcap$', '', oldest_pcap)

        jobs = Job.objects.filter(handle=handle)
        if jobs:
            jobs.delete()

    def run(self):
        criteria = self.job.criteria

        netshark = DeviceManager.get_device(criteria.netshark_device)

        self.export_name = str(path_to_class(netshark,
                                             criteria.netshark_source_name))

        source = netshark.get_capture_job_by_name(self.export_name)

        timefilter = TimeFilter(criteria.starttime, criteria.endtime)

        handle = Job._compute_handle(self.table, criteria)

        # check if pcaps directory exists, if not make the directory
        if not os.path.exists(PCAP_DIR):
            os.mkdir(PCAP_DIR)

        while self.all_pcap_size > settings.PCAP_SIZE_LIMIT:
            self.delete_oldest_pcap()

        self.filename = add_pcap_dir('%s.pcap' % handle)

        filters = ([BpfFilter(filt) for filt in self.table.options.filters]
                   or None)
        with netshark.create_export(
                source, timefilter, filters=filters,
                wait_for_data=self.table.options.wait_for_data,
                wait_duration=self.table.options.wait_duration) as e:
            self.download(e)

        return QueryComplete(pandas.DataFrame([dict(filename=self.filename)]))



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script performs backup and restores of configuration
to a NetShark appliance.
"""

import sys
import json
import getpass
from steelscript.common.datastructures import DictObject
from steelscript.netshark.core.app import NetSharkApp


class BackupApp(NetSharkApp):
    config_types = ['basic', 'auth', 'jobs', 'profiler_export', 'audit', 'protocols', 'users']
    protocol_resources = ['port_definitions', 'port_groups', 'l4_mappings',
                          'custom_applications', 'srt_ports', 'snmp',
                          'alerts']

    def add_options(self, parser):
        super(BackupApp, self).add_options(parser)
        parser.add_option('-f', '--filename',
                          help='Filename to use for backup/restore')
        parser.add_option('-b', '--backup', action='store_true',
                          help='Backup mode: store configuration to the specified file')
        parser.add_option('-r', '--restore', action='store_true',
                          help='Restore mode: apply configuration from the specified file')
        parser.add_option('-t', '--types',
                          help='Optional comma-separate list to limit the configuration '
                               'type(s) to backup/restore (options are %s).' %
                               (', '.join(self.config_types)))

    def main(self):
        if self.options.types is None:
            self.types = self.config_types
        else:
            self.types = self.options.types.split(',')
            for t in self.types:
                if t not in self.config_types:
                    raise ValueError("Invalid configuration type %s" % t)

        if self.options.filename is None:
            raise ValueError("Must specify backup/restore file")

        if self.options.backup:
            self.backup()
        elif self.options.restore:
            self.restore()
        else:
            raise RuntimeError("either backup mode (-b) or restore mode (-r) "
                               "must be specified")

    def backup(self):
        # First see if the file is there and if it has anything in it
        # already, parse it
        self.config = {}
        try:
            f = open(self.options.filename)
            self.config = json.load(f, object_hook=DictObject.create_from_dict)
            f.close()
        except Exception, e:
            print e, e.__class__

        print "Starting backup..."
        f = open(self.options.filename, 'w')

        for t in self.types:
            if t == 'basic':
                print "Backing up basic configuration..."
                self.config['basic'] = self.netshark.api.settings.get_basic()

            elif t == 'auth':
                print "Backing up authentication settings..."
                self.config['auth'] = self.netshark.api.settings.get_auth()

            elif t == 'audit':
                print "Backing up audit settings..."
                self.config['audit'] = self.netshark.api.settings.get_audit()

            elif t == 'profiler_export':
                print "Backing up netprofiler export settings..."
                self.config['profiler_export'] = self.netshark.api.settings.get_profiler_export()

            elif t == 'protocols':
                for resource in self.protocol_resources:
                    print "Backing up %s" % resource
                    self.config[resource] = getattr(self.netshark.api,
                                                    resource).get()

            elif t == 'jobs':
                print "Backing up jobs..."
                self.config['jobs'] = [j['config']
                                       for j in self.netshark.api.jobs.get_all()]

            elif t == 'users':
                print "Backing up users and groups..."
                self.config['users'] = self.netshark.api.users.get()
                self.config['groups'] = self.netshark.api.groups.get()

        json.dump(self.config, f, indent=4)
        print "Backup complete."

    def restore(self):
        print 'This operation will overwrite any existing configuration on your '
        print 'NetShark appliance.  Please verify that you want to continue.'
        result = raw_input('Continue?  [y/N] > ')
        if result not in ('Y', 'y'):
            print 'Okay, exiting.'
            sys.exit()

        f = open(self.options.filename)
        self.config = json.load(f, object_hook=DictObject.create_from_dict)
        f.close()

        print "Starting restore..."

        for t in self.types:
            if t == 'basic':
                print "Restoring basic configuration..."
                # XXX/demmer we may not want to do this since this
                # could change the IP address
                self.netshark.api.settings.update_basic(self.config['basic'])

            elif t == 'auth':
                print "Restoring authentication settings..."
                self.netshark.api.settings.update_auth(self.config['auth'])

                print "Reauthenticating to netshark..."
                self.netshark.reauthenticate()

            elif t == 'audit':
                print "Restoring audit settings..."
                self.netshark.api.settings.update_audit(self.config['audit'])

            elif t == 'profiler_export':
                print "Restoring netprofiler export settings..."
                self.netshark.api.settings.update_profiler_export(self.config['profiler_export'])

            elif t == 'protocols':
                for resource in self.protocol_resources:
                    print "Restoring %s" % resource
                    self.config[resource] = getattr(self.netshark.api,
                                                    resource).get()

            elif t == 'jobs':
                print "Restoring jobs..."

                # Don't blow away existing jobs with the same configuration
                job_config = self.netshark.api.jobs.get_all()

                config_by_name = {}
                for j in job_config:
                    config_by_name[j['config']['name']] = j['config']

                for new_job in self.config['jobs']:
                    if new_job.name in config_by_name:
                        if new_job == config_by_name[new_job.name]:
                            print "%s (skipped since already configured)..." % new_job.name
                        else:
                            print ("%s (ERROR: job exists with different configuration)..."
                                   % new_job.name)
                        continue

                    print "%s (configured)..." % new_job.name
                    self.netshark.api.jobs.add(new_job)

            elif t == 'users':
                print "Restoring groups..."
                # Don't blow away existing groups with the same configuration
                group_config = self.netshark.api.groups.get()

                config_by_name = {}
                for g in group_config:
                    config_by_name[g['name']] = g

                for new_group in self.config['groups']:
                    if new_group.name in config_by_name:
                        if new_group == config_by_name[new_group.name]:
                            print "%s (skipped since already configured)..." % new_group.name
                        else:
                            print ("%s (ERROR: group exists with different configuration)..."
                                   % new_group.name)
                        continue

                    print "%s..." % new_group.name
                    self.netshark.api.auth.groups.add(new_group)

                print "Restoring users..."
                user_config = self.netshark.api.users.get()

                config_by_name = {}
                for u in user_config:
                    config_by_name[u['name']] = u

                for new_user in self.config['users']:
                    if new_user.name in config_by_name:
                        if new_user == config_by_name[new_user.name]:
                            print "%s (skipped since already configured)..." % new_user.name
                        else:
                            print ("%s (ERROR: user exists with different configuration)..."
                                   % new_user.name)
                        continue

                    print "%s..." % new_user.name

                    # Need to coerce the structure for a new user
                    del new_user['is_admin']
                    del new_user['is_locked']

                    while True:
                        passwd = getpass.getpass("Enter password: ")
                        passwd2 = getpass.getpass("Enter password (again): ")

                        if passwd != passwd2:
                            print "ERROR: passwords do not match"
                            continue

                        new_user['password'] = passwd
                        break

                    self.netshark.api.auth.users.add(new_user)

        print "Restore complete."


if __name__ == '__main__':
    BackupApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from __future__ import absolute_import

import datetime
try:
    from steelscript.common.datetime import datetimeng
    datetimeng_available = True
except:
    datetimeng_available = False

from steelscript.common import timeutils


class TimeFilter(object):
    def __init__(self, start, end):
        if start is not None:
            self.start = timeutils.force_to_utc(start)
        else:
            self.start = 0

        if end is not None:
            self.end = timeutils.force_to_utc(end)
        else:
            self.end = 0

    def __unicode__(self):
        return '<TimeFilter - start: {}, end: {}>'.format(self.start, self.end)

    def to_dict(self):
        return {
            'type': 'TIME',
            'value': str(timeutils.datetime_to_nanoseconds(self.start)) +
            ', ' + str(timeutils.datetime_to_nanoseconds(self.end))
            }


class NetSharkFilter(object):
    def __init__(self, string):
        self.string = string

    def __unicode__(self):
        return '<{} - filter: {}>'.format(self.__class__, self.string)

    def to_dict(self):
        return {
            'type': 'SHARK',
            'value': self.string
            }


class BpfFilter(object):
    def __init__(self, string):
        self.string = string

    def __unicode__(self):
        return '<{} - filter: {}>'.format(self.__class__, self.string)

    def to_dict(self):
        return {
            'type': 'BPF',
            'value': self.string
            }



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
List all the key and column fields that the given NetShark appliance supports.
"""

import optparse

from steelscript.netshark.core.app import NetSharkApp
from steelscript.common.datautils import Formatter


class Command(NetSharkApp):
    help = 'List available NetShark extractor fields'

    def add_options(self, parser):
        super(Command, self).add_options(parser)
        group = optparse.OptionGroup(parser, "Column output options")
        group.add_option('--truncate', default=False, action='store_true',
                         help="truncate description column, don't wrap")
        group.add_option('-w', '--table-width', default=120,
                         help="max char width of table output, default: 120")
        parser.add_option_group(group)

    def main(self):
        headers = ['ID', 'Description', 'Type']
        data = [(f.id, f.description, f.type)
                for f in self.netshark.get_extractor_fields()]
        data.sort()
        Formatter.print_table(data,
                              headers,
                              padding=2,
                              max_width=int(self.options.table_width),
                              long_column=1,
                              wrap_columns=(not self.options.truncate))



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.




def str_to_filename(s):
    s = s.replace(":", "-")
    s = s.replace("?", "-")
    s = s.replace("*", "-")
    s = s.replace(";", "-")
    s = s.replace("{", "-")
    s = s.replace("}", "-")
    s = s.replace("&", "-")
    s = s.replace("\\", "-")
    s = s.replace(".", "-")
    s = s.replace("/", "-")
    return s

def parse_encoded_avg(s, precise=False):

    comp = s.split(':')

    if precise:
        return float(comp[0]) / float(comp[1])
    else:
        return int(comp[0]) / int(comp[1])

def value_to_int(s, precise=False):
    comp = s.split(':')

    if len(comp) == 1:
        if precise:
            return float(s)
        else:
            return int(s)
    elif len(comp) == 2:
        if precise:
            return float(comp[0]) / float(comp[1])
        else:
            return int(comp[0]) / int(comp[1])
    else:
        return 0



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a data source for querying data.

There are three parts to defining a data source:

* Defining column options via a Column class
* Defining table options via a DatasourceTable
* Defining the query mechanism via TableQuery

Note that you can define multiple Column and Table classes
in the same file, but only one TableQuery.  If you need
to define multiple types of queries, create multiple
files in this directory named accordingly.

"""

import copy
import logging
import datetime

import pandas

from steelscript.appfwk.apps.datasource.models import Table
from steelscript.appfwk.apps.jobs import QueryComplete, QueryContinue
from steelscript.appfwk.apps.jobs.models import Job
from steelscript.appfwk.apps.datasource.modules.analysis import \
    AnalysisTable, AnalysisQuery
from steelscript.appfwk.apps.devices.devicemanager import DeviceManager
from steelscript.appfwk.apps.devices.models import Device

logger = logging.getLogger(__name__)


class SharksTable(AnalysisTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _query_class = 'SharksQuery'

    @classmethod
    def create(cls, name, basetable, **kwargs):
        kwargs['related_tables'] = {'basetable': basetable}
        return super(SharksTable, cls).create(name, **kwargs)


class SharksQuery(AnalysisQuery):
    def analyze(self, jobs):
        criteria = self.job.criteria

        sharks_query_table = Table.from_ref(
            self.table.options.related_tables['basetable'])

        depjobs = {}

        # For every (shark, job), we spin off a new job to grab the data, then
        # merge everything into one dataframe at the end.
        for s in Device.objects.filter(module='netshark', enabled=True):
            shark = DeviceManager.get_device(s.id)

            for capjob in shark.get_capture_jobs():
                # Start with criteria from the primary table -- this gives us
                # endtime, duration and netshark_filterexpr.
                bytes_criteria = copy.copy(criteria)
                bytes_criteria.netshark_device = s.id
                bytes_criteria.netshark_source_name = 'jobs/' + capjob.name
                bytes_criteria.resolution = datetime.timedelta(0, 1)
                bytes_criteria.aggregated = True

                job = Job.create(table=sharks_query_table,
                                 criteria=bytes_criteria)

                depjobs[job.id] = job

        return QueryContinue(self.collect, depjobs)

    def collect(self, jobs=None):

        out = []
        for jid, job in jobs.iteritems():
            sharkdata = job.data()
            if sharkdata is not None:
                s = Device.objects.get(id=job.criteria.netshark_device)
                out.append([s.name,
                            s.host,
                            job.criteria.netshark_source_name[5:],
                            sharkdata['generic_bytes']])

        columns = ['name', 'host', 'capjob', 'bytes']
        df = pandas.DataFrame(out, columns=columns)
        return QueryComplete(df)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

from functools import partial

from _settings4 import Settings4, getted, \
    BasicSettingsFunctionality, ProfilerExport


class DPIResource(BasicSettingsFunctionality):
    def _get_by_name(self, name):
        for obj in self.data:
            if obj.get('name') == name:
                return obj
        return None

    def _get_by_priority(self, priority):
        obj = self.data[priority]
        if obj.get('priority') != priority:
            return None
        else:
            return obj

    def _port_string_to_port_list(self, port_string, protocol='TCP'):
        if port_string is not None:
            ports = [{'port_range': x.strip(), 'protocol': protocol}
                     for x in port_string.split(',')]
        else:
            ports = []
        return ports

    def _refresh_priorities(self):
        for i, obj in enumerate(self.data):
                obj['priority'] = i


class PortDefinitions(DPIResource):
    """Wrapper class and port definitions."""

    def __init__(self, api, srt_ports_api):
        super(PortDefinitions, self).__init__(api)
        self._srt_ports_api = srt_ports_api
        self._srtdata = None

    def get(self, force=False):
        if self._srtdata is None or force:
            self._srtdata = self._srt_ports_api.get()
        return super(PortDefinitions, self).get(force)

    @getted
    def save(self):
        super(PortDefinitions, self).save()
        self._srt_ports_api.update(self._srtdata)

    def _lookup_port(self, port):
        for port_obj in self.data:
            if port_obj['port'] == port:
                return port_obj
        return None

    @getted
    def add(self, name, port, protocol, srt):
        """Add a port definition

        :param str name: a name for the port

        :param int port: the port number

        :param str protocol: is the protocol the name is associated to
            can be `tcp` or `udp`

        :param bool srt: indicates if the port to be enabled as srt
            port

        """
        assert 0 < port < 65536
        assert srt is True or srt is False
        if protocol is not 'tcp' and protocol is not 'udp':
            raise ValueError('Protocol must be tcp or udp')

        port_obj = self._lookup_port(port)

        if port_obj is not None:
            if port_obj.get(protocol) is not None:
                raise ValueError('There is already a setting '
                                 'for port number {0}'.format(port))
            else:
                port_obj[protocol] = name
        else:
            self.data.append({protocol: name, 'port': port})

        if srt and protocol == 'tcp':
            if port not in self._srtdata:
                self._srtdata.append(port)

    @getted
    def remove(self, name, port):
        """Remove port identified by name and port number from the Port Definitions

        :param str name: a name for the port

        :param int port: the port number

        """
        assert 0 < port < 65536

        port_obj = self._lookup_port(port)

        if port_obj is None:
            raise ValueError('Port number {0} has no configuration '
                             'in the current netshark'.format(port))

        if port_obj.get('tcp') == name:
            del port_obj['tcp']
            if port in self._srtdata:
                self._srtdata.remove(port)

        if port_obj.get('udp') == name:
            del port_obj['udp']

        if port_obj.get('tcp') is None and port_obj.get('udp') is None:
            self.data.remove(port_obj)


class PortGroups(DPIResource):
    """Wrapper class and port group definitions."""

    @getted
    def add(self, name, tcp_ports=None, udp_ports=None, priority=None):
        # sort the list first by priority
        self.data.sort(key=lambda k: k['priority'])

        obj = self._get_by_name(name)

        if obj is not None:
            raise ValueError('A port group with the same name already exists')

        if priority is None:
            priority = len(self.data)

        tcp = self._port_string_to_port_list(tcp_ports, 'TCP')

        udp = self._port_string_to_port_list(udp_ports, 'UDP')

        self.data.insert(priority, {'name': name,
                                    'priority': priority,
                                    'ports': tcp+udp
                                    })

        self._refresh_priorities()

    @getted
    def remove(self, name=None, priority=None):
        """Remove a port group by name or by priority

        It accepts one of name or priority. If name and priority are issued
        it will remove the rule named `name` only if it matches `priority`

        :param str name: the name of the port group

        :param int priority: the priority of the port group

        """
        if name is None and priority is None:
            raise ValueError('name and priority cannot be both None')

        obj = None

        if name is not None:
            obj = self._get_by_name(name)

        if priority is not None:
            obj = self._get_by_priority(priority)

        if name is not None and priority is not None:
            if obj.get('name') != name:
                raise ValueError(
                    'Port group with priority {0} has a '
                    'different name than {1}'.format(priority, name)
                )

        if obj is not None:
            self.data.remove(obj)
        else:
            if name is not None and priority is None:
                raise ValueError('Impossible to find port group '
                                 'with name {0}'.format(name))
            if name is None and priority is not None:
                raise ValueError('Impossible to find port group '
                                 'with priority {0}'.format(priority))
            if name is not None and priority is not None:
                raise ValueError(
                    'Impossible to find port group '
                    'with name {0} and priority {1}'.format(name, priority)
                )


class L4Mapping(PortGroups):
    """Wrapper class around Layer 4 Mappings."""

    @getted
    def add(self, name, hosts, tcp_ports=None, udp_ports=None, priority=None):
        """Add a l4 mapping rule

        :param str name: the name of the rule

        :param str hosts: comma separated list of hosts with optional
                          subnet mask

        :param str tcp_ports: comma separated list of ports or port range

        :param str udp_ports: comma separated list of ports or port range

        """
        assert tcp_ports is not None and udp_ports is not None

        if priority is None:
            priority = len(self.data)

        obj = self._get_by_name(name)

        if obj is not None:
            raise ValueError('l4 mapping with name {0} '
                             'already exists'.format(name))

        tcp = self._port_string_to_port_list(tcp_ports, 'TCP')
        udp = self._port_string_to_port_list(udp_ports, 'UDP')

        self.data.insert(
            priority,
            {'name': name,
             'hosts': [x.strip() for x in hosts.split(',')],
             'priority': priority,
             'ports': tcp+udp
             }
        )

        self._refresh_priorities()

    @getted
    def remove(self, name=None, priority=None):
        """Remove a l4 mapping rule

        It accepts one of name or priority. If name and priority are issued
        it will remove the rule named `name` only if it matches `priority`

        :param str name: the name of the l4 mapping

        :param int priority: the priority of the l4 mapping
        """
        super(L4Mapping, self).remove(name, priority)


class CustomApplications(DPIResource):
    """Wrapper class around custom application definitions."""

    @getted
    def add(self, name, uri):
        """Add a custom application rule

        :param str name: the name of the rule

        :param str uri: a string representing a uri

        """

        obj = self._get_by_name(name)

        if obj is not None:
            raise ValueError('l4 mapping with name {0} '
                             'already exists'.format(name))

        self.data.append({'name': name, 'uri': uri})

    @getted
    def remove(self, name):
        """Remove a custom application rule

        :param str name: the name of the rule

        """
        obj = self._get_by_name(name)
        if obj is not None:
            self.data.remove(obj)
        else:
            raise ValueError('The rule with name {0} '
                             'does not exist'.format(name))


class ProfilerExport(ProfilerExport):
    """Wrapper class around Profiler Export"""

    def _lookup_profiler(self, address):
        for p in self.data['profilers']:
            if p['address'] == address:
                return p
        msg = ('No netprofiler with address {0} has been '
               'found in the configuration'.format(address))
        raise ValueError(msg)

    @getted
    def sync_dpi_with_profiler(self, address):
        """Helper method to select the current profiler to sync DPI with.

        This method modifes `self.data` to reflect the changes.  save() must
        be called to push the changes to the NetShark.

        """
        p = self._lookup_profiler(address)
        for prof in self.data['profilers']:
            if 'sync' in prof:
                del prof['sync']
        p['sync'] = {"sync_port_names": True,
                     "sync_port_groups": True,
                     "sync_layer4_mappings": True,
                     "sync_custom_applications": True}

    @getted
    def unsync_dpi_with_profiler(self, address):
        """Helper method to select the current profiler to sync DPI with.

        This method modifes `self.data` to reflect the changes.  save() must
        be called to push the changes to the NetShark.

        """
        p = self._lookup_profiler(address)
        if 'sync' in p:
            del p['sync']


class Alerts(BasicSettingsFunctionality):
    def test_snmp(self, obj):
        """Sends a test SNMP trap

        :param dict obj: object representing the trap to send.

        The ``obj`` must be of the form:

        .. code-block:: python

            {"address":"trap.riverbed.com","version":"V1","community":"test"}

        or

        .. code-block:: python

            {"address":"trap.riverbed.com","community":"public","version":"V2C"}

        or

        .. code-block:: python

            {
                "address": "trap.riverbed.com",
                "version": "V3",
                "username": "test",
                "engine_id": "testengine",
                "security_level": "AUTH_PRIVACY",
                "authentication": {"protocol":"MD5",
                                   "passphrase":"testpassword"},
                "privacy": {"protocol":"DES",
                            "passphrase":"testpassword"}
            }

        """
        return self._api.send_test_snmp(obj)

    def test_smtp(self, address, to_address, from_address, port=25):
        """Sends a test email via SMTP."""
        obj = {
            'smtp_server_address': address,
            'smtp_server_port': port,
            'to_address': to_address,
            'from_address': from_address
            }
        return self._api.send_test_smtp(obj)


class Settings5(Settings4):
    """Interface to various configuration settings on the netshark appliance.
    Version 5.0 API
    """

    def __init__(self, shark):
        super(Settings5, self).__init__(shark)
        self.port_definitions = PortDefinitions(shark.api.port_definitions,
                                                shark.api.srt_ports)
        self.port_groups = PortGroups(shark.api.port_groups)
        self.l4_mapping = L4Mapping(shark.api.l4_mappings)
        self.custom_applications = CustomApplications(
            shark.api.custom_applications
        )
        self.profiler_export = ProfilerExport(shark.api.settings)
        self.snmp = BasicSettingsFunctionality(shark.api.snmp)
        self.alerts = Alerts(shark.api.alerts)

        def raise_notimplemented_error(append):
            msg = ("This functionality has been replaced in this version of "
                   "NetShark with the DPI classes. "
                   "Please refer to the documentation for more information or "
                   "look at the instance of %s" % append)
            raise NotImplementedError(msg)

        # remove API 4.0 specific
        self.get_protocol_groups = partial(
            raise_notimplemented_error,
            'NetShark.settings.groups_definitions'
        )
        self.update_protocol_groups = partial(
            raise_notimplemented_error,
            'NetShark.settings.groups_definitions'
        )
        self.get_protocol_names = partial(
            raise_notimplemented_error,
            'NetShark.settings.l4_mapping'
        )
        self.update_protocol_names = partial(
            raise_notimplemented_error,
            'NetShark.settings.l4_mapping'
        )



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
Filters are used to reduce the traffic that is fed to NetShark
views, and are basic tools when doing data analysis with a NetShark
Appliance.  NetShark supports 4 main classes of filters: Time filters,
the native NetShark filters, BPF filters (also known as Wireshark
capture filters) and Wireshark display filters.
"""

from __future__ import absolute_import

from steelscript.common import timeutils
from steelscript.netshark.core._api4 import API4_0
from steelscript.netshark.core._api5 import API5_0
from steelscript.netshark.core import _filters4
from steelscript.netshark.core.netshark import FILTERS_MAP

FILTERS_MAP.update({
        API4_0: _filters4,
        API5_0: _filters4
        })


__all__ = [
    'TimeFilter',
    'NetSharkFilter',
    'WiresharkDisplayFilter',
    'BpfFilter'
]


class Filter(object):
    def __init__(self, *args, **kwargs):
        super(Filter, self).__init__(*args, **kwargs)

    def bind(self, shark):
        """Get the correct version of the filter based on the NetShark version
        """
        mod = FILTERS_MAP[shark.api.__class__]
        try:
            cls = getattr(mod, self.__class__.__name__)
        except AttributeError:
            return self

        return cls(**self.__dict__)


class TimeFilter(Filter):
    def __init__(self, start, end):
        """Create a TimeFilter representing a time range.

        :param datetime start: Start time

        :param datetime end: End time

        """
        super(TimeFilter, self).__init__()
        self.start = start
        self.end = end

    @classmethod
    def parse_range(cls, string):
        """Creata a TimeFilter based on human readable string.

        :param str string: time string

        The ``string`` parameter can be any time range string
        such as:

        * ``12:00 PM to 1:00 PM``
        * ``last 2 weeks``

        """
        (start, end) = timeutils.parse_range(string)
        return cls(start, end)


class NetSharkFilter(Filter):
    def __init__(self, string):
        """Create a filter in native NetShark syntax.

        :param str string: filter string

        """
        super(NetSharkFilter, self).__init__()
        self.string = string


class WiresharkDisplayFilter(Filter):
    def __init__(self, string):
        """Create a filter in Wireshark display syntax..

        :param str string: filter string

        """
        super(WiresharkDisplayFilter, self).__init__()
        self.string = string


class BpfFilter(Filter):
    def __init__(self, string):
        """Create a filter in BPF syntax.

        :param str string: filter string

        """
        super(BpfFilter, self).__init__()
        self.string = string



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""This files describes the class mapping for each NetShark protocol version
"""

from __future__ import absolute_import


class Classes(object):
    def __getattr__(self, attr):
        try:
            return self.__getattribute__(attr)
        except AttributeError:
            raise NotImplementedError('API version {0} is not available for this netshark version.'.format(attr))


class Classesv4(Classes):
    def __init__(self):
        #we must import here to let automatic registration
        #of classes working. If we import at module level
        #we will have multiple registration of classes between
        #different versions
        import steelscript.netshark.core._source4 as v4
        from steelscript.netshark.core import _view4, _fs, _settings4, _fields4

        self.Interface = v4.Interface4
        self.Job = v4.Job4
        self.Clip = v4.Clip4
        self.View = _view4.View4

        self.Directory = _fs.Directory4
        self.File = _fs.File4
        self.TraceFile = _fs.TraceFile4
        self.MultisegmentFile = _fs.MultisegmentFile4
        self.MergedFile = _fs.MergedFile4

        self.ExtractorField = _fields4.ExtractorField
        self.Settings = _settings4.Settings4

        self.Export = v4.Export4


class Classesv5(Classesv4):
    def __init__(self):
        super(Classesv5, self).__init__()
        import steelscript.netshark.core._source5 as v5
        from steelscript.netshark.core import _settings5
        self.Job = v5.Job5
        self.Interface = v5.Interface5
        self.Settings = _settings5.Settings5

def path_to_class(shark, path):
    mapping = dict(
        interfaces=shark.classes.Interface,
        fs=shark.classes.File,
        clips=shark.classes.Clip,
        jobs=shark.classes.Job
    )
    p = path.split('/', 1)
    if p[0] == 'jobs':
        # Try name, then id
        return (mapping[p[0]].get(shark, None, p[1]) or
                mapping[p[0]].get(shark, p[1]))
    else:
        return mapping[p[0]].get(shark, p[1])



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from __future__ import absolute_import

from steelscript.netshark.core import _interfaces
from steelscript.netshark.core._interfaces import loaded
from steelscript.netshark.core._exceptions import NetSharkException
from steelscript.common import datautils, timeutils


__all__ = ['Interface4', 'Clip4', 'Job4', 'Export4']


class Interface4(_interfaces._InputSource):
    """A physical interface packet source, that can be used for live packet
    capture. Interface objects are normally not instantianted directly, but
    are instead obtained by calling
    :py:func:`steelscript.netshark.core.netshark.NetShark.get_interfaces` or
    :py:func:`steelscript.netshark.core.netshark.NetShark.get_interface_by_name`."""
    def __init__(self, shark, data):
        super(Interface4, self).__init__(shark, data, shark.api.interfaces)
        self.id = self.data['id']

    def __repr__(self):
        return '<{0} {1} {2}>'.format(self.__class__.__name__,
                                      self.data['name'],
                                      self.data['type'])

    def __str__(self):
        return '{0}'.format(self.data['name'])

    @property
    def name(self):
        """The interface device name."""
        return self.data['name']

    @property
    def description(self):
        """The interface description."""
        return self.data['description']

    def is_live(self):
        """Can be used to determine if a source object is live or offline.
        The result is always True for an Interface4."""
        return True

    @property
    def source_path(self):
        return 'interfaces/{0}'.format(self.id)

    @property
    def source_options(self):
        return {}

    @classmethod
    def get_all(cls, shark):
        interfaces = shark.api.interfaces.get_all()
        return [cls(shark, data) for data in interfaces]

    def update(self):
        """Update current object with the values from the server
        """
        data = self._api.get_details(self.id)
        super(Interface4, self).update(data)


class Clip4(_interfaces.Clip):
    """A trace clip packet source.
    These objects are returned by `NetShark.get_clips`."""
    def __init__(self, shark, data):
        super(Clip4, self).__init__(shark, data, shark.api.clips)
        self.id = self.data['id']

    def __str__(self):
        return 'clips/' + self.id

    def _ensure_loaded(self):
        if self.data is None or len(self.data) == 1:
            self._load()

    def _load(self):
        self.data = self._api.get_details(self.id)

    @property
    def source_path(self):
        """Return the path of a Clip used for the internal netshark representation
        """
        return 'clips/{0}'.format(self.id)

    @property
    def source_options(self):
        return {}

    @property
    @loaded
    def description(self):
        """Returns the description of the clip
        """
        return self.data['config']['description']

    @property
    @loaded
    def size(self):
        """Returns the size of the clip
        """
        return self.data['status']['estimated_size']

    def delete(self):
        """Erase the clip from netshark
        """
        self._api.delete(self.id)
        self.id = None

    @classmethod
    def get_all(cls, shark):
        """Get the complete list of trace files on given a NetShark.

        ``netshark`` is an ``steelscript.netshark.core.netshark.NetShark`` object

        Returns a list of ``TraceFile`` objects
        """
        return [cls(shark, data) for data in shark.api.clips.get_all()]

    @classmethod
    def add(cls, shark, job, filters, description, locked=True):
        """Create a new clip given a NetShark connection and a job handle.

        `netshark` is a NetShark object

        `description` will be associated with the new clip.
        (The description is shown near the clip in grey in Pilot)

        `job` is the capture job to use

        `filters` is the list of filters to associate to the clip.
        In order for the clip to be valid, there must be at list one
        time filter in this list

        Returns a trace clip object for the new clip.
        """
        config = {
            'job_id': job.id,
            'filters': [filt.bind(shark) for filt in filters],
            'description': description,
            }
        ret = shark.api.clips.add(config)
        clip = cls(shark, ret)
        if locked:
            clip.locked(True)
        return clip

    @loaded
    def locked(self, state=None):
        """Set or retrieve locked state of a clip
        """
        pass

    def download(self, path=None):
        """Download the Clip packets to a file.
        If path is None packets will be exported to a temporary file.
        A file object that contains the packets is returned.
        """
        return open(self._api.get_packets(self.id, path), 'rb')


class Job4(_interfaces.Job):
    """A capture job packet source. These objects are normally not
    instantiated directly, but are instead obtained by calling
    `NetShark.get_capture_jobs` or `NetShark.get_capture_job_by_name`.
    """

    def __init__(self, shark, data):
        super(Job4, self).__init__(shark, data, shark.api.jobs)
        self.id = self.data['id']
        self.index_enabled = True
        self.data = data

    def _ensure_loaded(self):
        if self.data is None or len(self.data) == 1:
            self._load()

    def _load(self):
        self.data = self._api.get_details(self.id)

    @loaded
    def __repr__(self):
        return '<{0} {1} on {2}>'.format(
            self.__class__.__name__,
            self.name,
            self.data['config']['interface_description'])

    @loaded
    def __str__(self):
        return '{0}'.format(self.data['config']['name'])

    @property
    def source_path(self):
        return 'jobs/{0}'.format(self.id)

    @property
    @loaded
    def source_options(self):
        if not self.index_enabled:
            return {'disable_index': True}
        else:
            return {}

    def is_live(self):
        """Can be used to determine if a source object is live or offline.
        The result is always False for a CaptureJob."""
        return False

    @property
    @loaded
    def name(self):
        """The capture job name"""
        return self.data['config']['name']

    @property
    @loaded
    def size_on_disk(self):
        """The capture job actual size, corresponding to the one shown by the
        NetShark UI shows."""
        return self.get_status()['packet_size']

    @property
    @loaded
    def size_limit(self):
        """The capture job maximum size, corresponding to the one shown by the
        NetShark UI shows."""
        return self.data['config']['packet_retention']['size_limit']

    @property
    def packet_start_time(self):
        return self.get_status()['packet_start_time']

    @property
    def packet_end_time(self):
        return self.get_status()['packet_end_time']

    @property
    def interface(self):
        """an :py:class:`Interface` object, representing the interface used as a packet
        source for this job."""
        interfaces = Interface4.get_all(self.shark)
        for interface in interfaces:
            if self.data['config']['interface_name'] == interface.id:
                return interface
        ValueError('{0} interface not found'.format(self.data['config']['interface_description']))

    @property
    def handle(self):
        """The internal capture job handle. The handle is sometimes required
        for advanced operations on the job.
        """
        return self.data['id']

    @classmethod
    def get_all(cls, shark):
        """Get the complete list of capture jobs on given a NetShark.

        ``netshark`` is an :py:class:`steelscript.netshark.core.netshark.NetShark` object

        Returns a list of ``CaptureJob`` objects

        size_on_disk, keep_size, packet_start_time,
                 packet_end_time,
        """
        jobs = []
        for j in shark.api.jobs.get_all():
            jobs.append(cls(shark, j))
        return jobs

    @classmethod
    def create(cls, shark, interface, name,
               packet_retention_size_limit,
               packet_retention_packet_limit=None,
               packet_retention_time_limit=None,
               bpf_filter=None,
               snap_length=65525,
               indexing_size_limit=None,
               indexing_synced=False,
               indexing_time_limit=None,
               start_immediately=True,
               requested_start_time=None,
               requested_stop_time=None,
               stop_rule_size_limit=None,
               stop_rule_packet_limit=None,
               stop_rule_time_limit=None):
        """Create a new capture job"""

        def _calc_size(size, total):
            if isinstance(size, str) and size[-1] == '%':
                size = total * int(size[:-1]) / 100
            elif not isinstance(size, (int, long)) and size is not None:
                size = datautils.human2bytes(size)
            return size

        stats = shark.get_stats()
        packet_storage_total = stats['storage']['packet_storage']['total']
        index_storage_total = stats['storage']['os_storage']['index_storage']['total']

        packet_retention_size_limit = _calc_size(packet_retention_size_limit, packet_storage_total)
        stop_rule_size_limit = _calc_size(stop_rule_size_limit, packet_storage_total) if stop_rule_size_limit else None
        indexing_size_limit = _calc_size(indexing_size_limit, index_storage_total) if indexing_size_limit else None

        packet_retention_packet_limit = int(packet_retention_packet_limit) if packet_retention_packet_limit else None
        stop_rule_size_limit = int(stop_rule_size_limit) if stop_rule_size_limit else None
        stop_rule_packet_limit = int(stop_rule_packet_limit) if stop_rule_packet_limit else None
        snap_length = int(snap_length) if snap_length else 65535
        indexing_size_limit = int(indexing_size_limit) if indexing_size_limit else None

        if indexing_time_limit:
            indexing_time_limit = int(timeutils.timedelta_total_seconds(indexing_time_limit))
        if packet_retention_time_limit:
            packet_retention_time_limit = int(timeutils.timedelta_total_seconds(packet_retention_time_limit))
        if stop_rule_time_limit:
            stop_rule_time_limit = int(timeutils.timedelta_total_seconds(stop_rule_time_limit))

        if requested_start_time:
            requested_start_time = requested_start_time.strftime(cls._timefmt)
        if requested_stop_time:
            requested_stop_time = requested_stop_time.strftime(cls._timefmt)

        jobrequest = {'interface_name': interface.id}
        jobrequest['name'] = name

        if packet_retention_size_limit or packet_retention_packet_limit or packet_retention_time_limit:
            jobrequest['packet_retention'] = dict()
            if packet_retention_size_limit:
                jobrequest['packet_retention']['size_limit'] = packet_retention_size_limit
            if packet_retention_packet_limit:
                jobrequest['packet_retention']['packet_limit'] = packet_retention_packet_limit
            if packet_retention_time_limit:
                jobrequest['packet_retention']['time_limit'] = packet_retention_time_limit
        if bpf_filter:
            jobrequest['bpf_filter'] = bpf_filter
        if requested_start_time:
            jobrequest['requested_start_time'] = requested_start_time
        if requested_stop_time:
            jobrequest['requested_stop_time'] = requested_stop_time
        if stop_rule_size_limit or stop_rule_packet_limit or stop_rule_time_limit:
            jobrequest['stop_rule'] = dict()
            if stop_rule_size_limit:
                jobrequest['stop_rule']['size_limit'] = stop_rule_size_limit
            if stop_rule_packet_limit:
                jobrequest['stop_rule']['packet_limit'] = stop_rule_packet_limit
            if stop_rule_time_limit:
                jobrequest['stop_rule']['time_limit'] = stop_rule_time_limit
        if snap_length:
            jobrequest['snap_length'] = int(snap_length)

        # Indexing logic table
        # case   size   synced    time    valid?
        # 1      None   False     None    valid - index is disabled
        # 2      None   False     int     error
        # 3      None   True      None    error
        # 4      None   True      int     error
        # 5      int    False     None    valid
        # 6      int    False     int     valid
        # 7      int    True      None    valid
        # 8      int    True      int     valid

        if (indexing_synced or indexing_time_limit) and not indexing_size_limit:
            raise NetSharkException(
                'indexing_size_limit must be specified '
                'with indexing_synced or indexing_time_limit')
        elif indexing_size_limit:
            jobrequest['indexing'] = dict()
            jobrequest['indexing']['size_limit'] = indexing_size_limit
            if indexing_synced:
                jobrequest['indexing']['synced'] = indexing_synced
            if indexing_time_limit:
                jobrequest['indexing']['time_limit'] = indexing_time_limit

        jobrequest['start_immediately'] = start_immediately

        job_id = shark.api.jobs.add(jobrequest)

        job = cls(shark, job_id)
        return job

    def start(self):
        """Start a job in the NetShark appliance
        """
        self._api.state_update(self.id, {'state': "RUNNING"})

    def stop(self):
        """Stop a job in the NetShark appliance
        """
        self._api.state_update(self.id, {'state': "STOPPED"})

    def clear(self, restart=False):
        """Clear data in the NetShark appliance
        """
        self._api.state_update(self.id, {'state': 'STOPPED',
                                         'clear_packets': True})
        if restart:
            self.start()

    def delete(self):
        """Delete job from the NetShark appliance
        """
        try:
            self.stop()
        except:
            pass
        self._api.delete(self.id)

    def add_clip(self, filters, description, locked=True):
        """
        Create a new trace clip under this job.

        `filters` limit which packets from the clip should go into
        this clip.  Since all packets in the new clip will be kept on
        disk if the clip is locked, this field typically includes
        time filters to limit the clip to a fixed time interval.

        `description` is a name for the clip, it is shown next to the
        clip in the Pilot user interface.

        If `locked` is True, packets in the new clip will not be deleted
        from disk as long as the clip exists.  Note that locking packets
        in trace clips necessarily reduces the amount of disk capacity
        for existing capture jobs.

        Returns a trace clip object representing the new clip.
        """
        return Clip4.add(self.shark, self, filters, description, locked)

    def download(self, path=None):
        """Download the Job packets to a path.
        If path is None packets will be exported to a temporary file.
        A file object that contains the packets is returned.
        """
        return open(self._api.get_packets(self.id, path), 'rb')

    def get_state(self):
        """Return the state of the job (e.g. RUNNING, STOPPED)"""
        return self.get_status()['state']

    def get_status(self):
        """Return status information about the capture job."""
        return self._api.get_status(self.id)

    def get_stats(self):
        """Return statistics about the capture job."""
        return self._api.get_stats(self.id)

    def get_index_info(self):
        """Return statistics about the capture job index."""
        return self._api.get_index(self.id)


class Export4(_interfaces.Export):
    pass



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This simple script creates a trace clip on the first available job, with the
following properties:
 - time range is the last 30 minutes
 - the clip contains only packets for the IP address 192.168.0.1
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.netshark.core.filters import TimeFilter, NetSharkFilter


class CreateClipApp(NetSharkApp):
    def main(self):
        # Get the list of jobs
        jobs = self.netshark.get_capture_jobs()
        if len(jobs) == 0:
            print ("No jobs on the appliance, you can create jobs using the "
                   "'create_job.py' and start/stop them using the "
                   "'control_job.py' script.")
            return 0

        # Pick the first job
        job = jobs[0]
        print 'creating a 30 minutes clip on job {0}'.format(job.name)

        # set the filters
        filters = (
            # Time filter: keep the last 30 minutes
            TimeFilter.parse_range("last 30 m"),

            # IP address filter: keep only 192.168.0.1
            NetSharkFilter('ip.src="192.168.0.1"')
        )

        # Create the clip
        clip = job.add_clip(filters, "a_test_clip")

        print 'Clip with the following properties created:'
        print ''
        print 'ID: %s' % clip.id
        print 'Description: %s' % clip.description
        print 'Source Path: %s' % clip.source_path
        print 'Size: %s' % clip.size


if __name__ == '__main__':
    CreateClipApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.common.datastructures import DictObject


class ExtractorField(DictObject):
    @classmethod
    def get_all(cls, shark):
        return [cls(field) for field in shark.api.info.get_fields()]



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.




class NetSharkAPIVersions(object):
    CURRENT = "5.0"
    LEGACY = ["4.0"]

class APIGroup(object):
    '''Simple wrapper class for a group of related API functions'''
    def __init__(self, uri_prefix, shark):
        self.uri_prefix = uri_prefix
        self.shark = shark

class APITimestampFormat(object):
    '''Encapsulation of the various options supported for absolute
    timestamp encoding.'''
    NANOSECOND             = "ns number"
    NANOSECOND_STR         = "ns string"
    MICROSECOND            = "us number"
    MICROSECOND_STR        = "us string"
    MILLISECOND            = "ms number"
    MILLISECOND_STR        = "ms string"
    SECOND                 = "s number"
    SECOND_STR             = "s string"
    SECOND_NANOSECOND      = "s.ns number"
    SECOND_NANOSECOND_STR  = "s.ns string"
    SECOND_MICROSECOND     = "s.us number"
    SECOND_MICROSECOND_STR = "s.us string"
    SECOND_MILLISECOND     = "s.ms number"
    SECOND_MILLISECOND_STR = "s.ms string"



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script demonstrates how to use the View class to do two
simple and common tasks:
1. enumerate all the existing views on a netshark (the -l command line option)
2. attach to an existing view, extract a section of data from it, and
   write it to a CSV file
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.netshark.core import viewutils


class ReadView(NetSharkApp):

    def add_options(self, parser):
        super(ReadView, self).add_options(parser)
        parser.add_option('-l', '--listviews', action="store_true",
                          default=False, help='print a list of running views')
        parser.add_option('-v', '--viewname', default=None,
                          help='view name to read')
        parser.add_option('-f', '--filename', default=None,
                          help='write CSV file to given file name')
        parser.add_option('-o', '--output', default=None,
                          help='retrieve only the specified output field')

    def validate_args(self):
        super(ReadView, self).validate_args()
        if self.options.listviews:
            # list views ignores other options
            return

        if not self.options.viewname:
            self.parser.error('view name required, use "-v" option to specify')

    def _do_one_output(self, output):
        legend = output.get_legend()
        data = output.get_iterdata()

        # If the -f option has been used, we save the data to disk as csv,
        # otherwise we print it to the screen.
        if self.options.filename is not None:
            viewutils.write_csv(self.options.filename, legend, data)
        else:
            viewutils.print_data(legend, data)

    def main(self):
        if self.options.listviews:
            # If -l is specified, we show the list of views running on the
            # appliance, and the outputs for each of them,
            views = self.netshark.get_open_views()
            if len(views) == 0:
                print 'there are no views!'
            for view in views:
                print view.handle
                for output in view.all_outputs():
                    print '  {0}'.format(output.id)
            return

        # Retrieve details about a specific view
        try:
            view = self.netshark.get_open_view_by_handle(self.options.viewname)
        except KeyError:
            print 'cannot find view {0}'.format(self.options.viewname)
            return -1

        # A view can have one or more outputs. Each output corresponds to one
        # of the charts that can be seend when opening the view in Pilot.
        # If an output name is specified, we will print (or save) the data for
        # that output only.
        # Otherwise, we merge all the outputs and print them all.
        if self.options.output is not None:
            self._do_one_output(view.get_output(self.options.output))
        else:
            outputs = view.all_outputs()
            if len(outputs) > 1:
                try:
                    mixer = viewutils.OutputMixer()
                    for o in outputs:
                        mixer.add_source(o, '')
                    self._do_one_output(mixer)
                    outputs = []
                except NotImplementedError:
                    pass

            # we can end up here if there is just one output or if
            # we can't mix multiple outputs
            if len(outputs) > 0:
                for o in outputs:
                    print 'Output %s' % o.id
                    self._do_one_output(o)


if __name__ == '__main__':
    ReadView().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from _api_helpers import APIGroup, APITimestampFormat
from _api4 import API4_0
import urllib


class API5Group(APIGroup):
    base_headers = {}

    def request(self, urlpath, method, data=None,
                timestamp_format=APITimestampFormat.NANOSECOND,
                params=None,
                custom_headers=None):
        """Issue the given API request using either JSON or XML
        (dictated by the as_json parameter)."""
        self.add_base_header('X-RBT-High-Precision-Timestamp-Format',
                             timestamp_format)

        headers = dict(self.base_headers)
        if isinstance(custom_headers, dict):
            headers.update(custom_headers)

        # we are dealing with a url so let's sanitize it.  this may break code
        # but at least prevents steelscript to send insane urls to the server
        # define insane: url that contains spaces
        urlpath = urllib.quote(urlpath)

        return self.shark.conn.json_request(method, self.uri_prefix + urlpath,
                                            data, params, headers)

    def add_base_header(self, key, value=""):
        if isinstance(key, basestring):
            self.base_headers[key] = value

    def remove_base_header(self, key):
        if isinstance(key, basestring):
            self.base_headers.pop(key, None)


class PortDefinitions(API5Group):
    def get(self):
        """Get the configuration from the server"""
        return self.request('/port_names', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/port_names', 'PUT', data)


class PortGroups(API5Group):
    def get(self):
        """Get the configuration from the server"""
        return self.request('/port_groups', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/port_groups', 'PUT', data)


class L4Mappings(API5Group):
    def get(self):
        """Get the configuration from the server"""
        return self.request('/layer4_mappings', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/layer4_mappings', 'PUT', data)


class CustomApplications(API5Group):
    def get(self):
        """Get the configuration from the server"""
        return self.request('/custom_applications', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/custom_applications', 'PUT', data)


class SrtPorts(API5Group):
    def get(self):
        """Get the configuration from the server"""
        return self.request('/srt_ports', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/srt_ports', 'PUT', data)


class SNMP(API5Group):
    def get(self):
        """Get the configuration for the SNMP from the server"""
        return self.request('/snmp', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/snmp', 'PUT', data)


class Alerts(API5Group):
    def get(self):
        """Gets the configuration for the alerts from the server"""
        return self.request('/notification', 'GET')

    def update(self, data):
        """Updates the configuration to the server"""
        return self.request('/notification', 'PUT', data)

    def send_test_snmp(self, data):
        """Tries the SNMP configuration"""
        return self.request('/notification/send_test_trap.json', 'POST', data)

    def send_test_smtp(self, data):
        """Tries the SMTP configuration"""
        return self.request('/notification/send_test_mail.json', 'POST', data)


class API5_0(API4_0):
    version = '5.0'
    common_version = '1.0'

    def __init__(self, netshark):
        super(API5_0, self).__init__(netshark)
        self.port_definitions = PortDefinitions(
            '/api/shark/' + self.version + '/definitions', netshark
        )
        self.port_groups = PortGroups(
            '/api/shark/' + self.version + '/definitions', netshark
        )
        self.l4_mappings = L4Mappings(
            '/api/shark/' + self.version + '/definitions', netshark
        )
        self.custom_applications = CustomApplications(
            '/api/shark/' + self.version + '/definitions', netshark
        )
        self.srt_ports = SrtPorts(
            '/api/shark/' + self.version + '/definitions', netshark
        )
        self.snmp = SNMP(
            '/api/shark/' + self.version + '/settings', netshark
        )
        self.alerts = Alerts(
            '/api/shark/' + self.version + '/settings', netshark
        )

__all__ = ['API5_0']



# Copyright (c) 2017 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.raw as raw

from steelscript.netshark.appfwk.datasources.netshark_msa import \
    MSADownloadTable

#
# Download PCAPs from two NetSharks and upload to a third for MSA analysis
#

report = Report.create('NetShark MSA Download', position=10)

report.add_section()

t = MSADownloadTable.create(name='MultiSegment Table')
t.add_column('results', label='Results', iskey=True)

report.add_widget(raw.TableWidget, t, 'MSA Results', width=12, height=400)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""File and Directory classes encapsulate everything needed to handle
files and directory in NetShark
"""

from __future__ import absolute_import

import os

from steelscript.netshark.core._exceptions import NetSharkException
from steelscript.netshark.core._interfaces import loaded, _InputSource
import datetime

class _FSResource(object):
    """This class contains methods to manage both Files and Directories.
    """
    data = None
    shark = None

    def __init__(self, shark, data):
        """Create a resource instance and it fetches all
        the  from remote. This will raise an exception if the remote
        directory  does  not exist. If the caller provides a data
        dictionary  it  just  sets  it without calling the API,
        this optimization is used by the list method.
        """
        path = data['id']
        assert shark is not None
        assert path is not None and len(path) > 0

        # Here it saves the netshark reference so the caller does not have to
        # provide it for every call
        self.shark = shark
        #detect if we have a full dictionary of the resource or just its id
        if len(data) < 2 or 'created' not in data:
            # On the server side (and on the low level steelscript fs api) we
            # have two command to get resources details. 'get_all' returns
            # info about the root directory content, 'get_details' returns
            # info about all the other directories and files. The two
            # commands have different outputs on the low level and mostly of
            # the data available in the output of the second one are missing
            # in the first one output (id, creation date, modified date,
            # file # list) so this method have to set some so that the user
            # can use the directory reference. Also notice that whatever
            # action the user will try to do with the root directory
            # different from listing sub-dirs content will fail (e.g.
            # renaming a directory, creating a sub-dir, moving a directory,
            # uploading a trace file). Only NetShark can control the root
            # directory
            if path == self.shark._file_separator:
                self.data = self._get_root_details()
            else:
                self.data = self.shark.api.fs.get_details(path, details=True)
            assert isinstance(self.data, dict)

        else:
            assert isinstance(data, dict)
            self.data = data

    def _get_root_details(self):

        root_dirs = self.shark.api.fs.get_all(details=True, recursive=True)

        data = {}
        data["id"] = self.shark._file_separator

        # It would not make sense setting 0 here since that date would be
        # wrong
        data["created"] = None
        data["modified"] = None
        data["files"] = []
        data["dirs"] = []

        for current_dir_details in root_dirs:
            data["dirs"].append(Directory4(self.shark, current_dir_details["id"]))

        return data

    def remove(self):
        """Remove a remote resource and its details info
        """
        assert self.shark is not None
        assert self.data is not None

        self.shark.api.fs.delete(self.data["id"])
        self.data = None

    def move(self, new_path):
        """Move or rename a resource. 'new_path' should include the
        whole new path. If 'new_path' already exists and is a
        directory, then the resource is moved into that directory. If
        'new_path' does not exist, then the resource is renamed to the
        specified path."""
        assert self.shark is not None
        assert self.data is not None
        assert new_path is not None and len(new_path) > 0

        json_request = {"destination": new_path}
        self.shark.api.fs.move(self.data["id"], json_request)

        self._update_details(new_path)

    def get_info(self):
        """
        Retrieve info about a remote fs resource
        """
        assert self.shark is not None
        assert self.data is not None

        self.data = self.shark.api.fs.get_details(self.data["id"], details=True)
        return self.data

    @classmethod
    def exists(cls, shark, path):
        """Returns true if the resource exists, false otherwise
        """

        assert shark is not None
        assert path is not None and len(path) > 0

        try:
            shark.api.fs.get_details(path)
        except Exception:
            #vampolo: Exception is too generic, change it
            return False
        else:
            return True

    def _update_details(self, path):
        """Update the details object
        """
        assert self.shark is not None
        assert self.data is not None

        self.data = self.shark.api.fs.get_details(path, details=True)

    def __str__(self):
        return self.data["id"]

    def _ensure_loaded(self):
        # XXX we have a skeleton data dictionary even if the details
        # haven't been loaded yet.  look for a key that we must get
        # from the server: created
        if "created" not in self.data:
            self._load()

    def _load(self):
        self.data = self.shark.api.fs.get_details(self.data["id"], details=True)

    @property
    @loaded
    def path(self):
        """The file path
        """
        return self.data["id"]

    def is_live(self):
        """Can be used to determine if a source object is live or offline.
        The result is always False for a FS Resouce."""
        return False

    @property
    @loaded
    def created(self):
        """ Creation time timestamp
        """
        creation_time = None
        if self.data["created"] is not None:
            creation_time = datetime.datetime.fromtimestamp(self.data["created"])
        return creation_time

    @property
    @loaded
    def modified(self):
        """ Modification time timestamp
        """
        modification_time = None
        if self.data["modified"] is not None:
            modification_time = datetime.datetime.fromtimestamp(self.data["modified"])

        return modification_time

class Directory4(_FSResource):

    def __init__(self, shark, path):
        if not isinstance(path, dict):
            d = dict(id=path)
        else:
            d = path
        super(Directory4, self).__init__(shark, d)

    def __repr__(self):
        return "<Directory path='{0}'>".format(self.data['id'])

    def _process_json_directories(self, json_data):
        """Process a json representing the directories
        This encapsulate the output of the fs get_details call to
        Flyscript objects
        """
        dir_list = list()
        file_list = list()
        for current_dir_details in json_data["dirs"]:
                dir_list.append(Directory4(self.shark, current_dir_details))

        for current_file_details in json_data["files"]:
            cls = File4._get_child_class(current_file_details)
            new_file = cls(self.shark, current_file_details)
            file_list.append(new_file)
        return dir_list, file_list

    @classmethod
    def create(cls, shark, path):
        """Create the directory and it returns a reference to it
        """
        error_msg = "An error occurred creating the directory "
        assert shark is not None
        assert path is not None and len(path) > 0

        # The API requires the directory name and the directory
        # parent dir as two separated parameters
        src_dir, dir_name = os.path.split(path)

        #Remove leading and trailing white space from the new resource name
        dir_name = dir_name.strip()

        if src_dir is not None and dir_name is not None and \
               len(src_dir) > 0 and len(dir_name) > 0:

            headers = { 'Content-Disposition' : dir_name,
                        'Content-Type' : 'application/x-shark-directory'
            }
            shark.api.fs.upload_raw(src_dir, None, headers)

            new_path = src_dir + shark._file_separator + dir_name
            return cls(shark, new_path)

        raise NetSharkException(error_msg + path + \
                             ": the path provided is not correct")

    def create_subdir(self, dir_name):
        """Create a sub-directory in the current directory and
        returns a reference to it. The 'dir_name' parameter should
        only include the directory name.
        """
        assert self.shark is not None

        complete_path = self.data["id"] + self.shark._file_separator + dir_name
        new_dir_ref = Directory4.create(self.shark, complete_path)
        self._update_details(self.data["id"])
        return new_dir_ref

    def list(self, recursive = False):
        """ List the directory content and return an array with the
        sub-directories and an array with the files
        """
        assert self.shark is not None

        #the get_details call has different result on / than on any
        #other directory, so we should take care of it
        res = self.shark.api.fs.get_details(
            self.data["id"], details=True, recursive=recursive)

        dir_list = []
        file_list = []

        if self.data['id'] == self.shark._file_separator:
            #we are in the root case, res is thus a list of directories
            for directory in res:
                dir_list.append(Directory4(self.shark, directory))
            #no need to check for files, they are not in the root
            #directory by design
        else:
            dir_list, file_list = self._process_json_directories(res)
        return dir_list, file_list

    def upload_trace_file(self, remote_file_name, local_path):
        """Upload a trace file and it returns a reference  to  it.
        'local_path' is the local file path, 'remote_file_name'
        contains only the file name
        """
        assert self.shark is not None

        complete_remote_path = self.data["id"] + self.shark._file_separator + remote_file_name
        return TraceFile4.upload(self.shark, complete_remote_path, local_path)


    def _walk(self, data, dirs):
        """Helper for walk in order to use recursive function without
        making subsequent REST calls
        """
        def data_lookup(data, id):
            """simply lookup a specific directory object
            by id from a json dictionary
            """
            if isinstance(data, list):
                #this is the case in which we are doing a
                #get_details of / that returns a list
                #so we check the id of the returned objects
                for item in data:
                    if item['id'] == id:
                        return item
            else:
                #this is the case of a get_details of a directory
                #so we look at the subdirectories
                for dir in data['dirs']:
                    if dir['id'] == id:
                        return dir


        for dir in dirs:
            sub_data = data_lookup(data, dir.data['id'])
            sub_dirs, sub_files = self._process_json_directories(sub_data)

            yield dir.data['id'], sub_dirs, sub_files

            for x in self._walk(sub_data, sub_dirs):
                yield x

    def walk(self):
        """Generate the file names in a directory tree by walking the tree in a top-down way.
        For each directory in the tree rooted at directory top (including top itself),
        it yields a 3-tuple (dirpath, dirnames, filenames).
        """
        res = self.shark.api.fs.get_details(
            self.data['id'], details=True, recursive=True)

        dirs = list()
        files = list()

        if self.data['id'] == self.shark._file_separator:
            for dir in res:
                dirs.append(Directory4(self.shark, dir))
        else:
            dirs, files = self._process_json_directories(res)

        yield self.data['id'], dirs, files

        for x in self._walk(res, dirs):
            yield x


class File4(_FSResource, _InputSource):

    #this dictionary stores the string_type:class
    #for each supported File child
    types = dict()

    def __init__(self, shark, data):
        super(File4, self).__init__(shark, data)
        _InputSource.__init__(self, shark, data, shark.api.fs)

    @property
    def source_path(self):
        if self.data['id'][0] == self.shark._file_separator:
            return 'fs' + self.data['id']
        else:
            return 'fs' + self.shark._file_separator + self.data['id']

    @property
    def source_options(self):
        return {}

    @classmethod
    def get_all(cls, shark):
        res = list()

        for directory in shark.api.fs.get_all(recursive=True):
            cls._get_all(shark, directory, res)

        return res

    @classmethod
    def _get_all(cls, shark, dirs, lst):
        for dir in dirs['dirs']:
            cls._get_all(shark, dir, lst)

        for file in dirs['files']:
            clss = cls._get_child_class(file)
            lst.append(clss(shark, file))

        return lst

    @classmethod
    def _get_child_class(cls, fi):
        """Given a json representation of a file returns
        the correct class that represent that file
        """
        if fi['type'] in cls.types:
            clss = cls.types[fi['type']]
        else:
            clss = File4
        return clss

    @classmethod
    def get(cls, shark, path):
        assert shark is not None
        assert path is not None and len(path) > 0

        data = shark.api.fs.get_details(path, details=True)
        clss = cls._get_child_class(data)
        return clss(shark, data)

    def download(self, local_path):
        """Download a trace file. 'local_path' is the local file path
        """
        assert self.shark is not None

        self.shark.api.fs.download(self.data["id"], local_path)

    @property
    @loaded
    def size(self):
        """The resource disk size
        """
        return self.data["size"]

    @property
    def link_type(self):
        """The file link type (e.g. ethernet, 802.11...), as a lipbap DLT
        (see `<http://www.tcpdump.org/pcap3_man.html>`_)"""
        return self.data["link_type"]

    def copy(self, new_path):
        """Copy a file. 'new_path' should can be the whole new
        path or the destination folder.
        """
        assert self.shark is not None
        assert self.data is not None

        json_request = {"destination": new_path}
        self.shark.api.fs.copy(self.data["id"], json_request)

class FileMeta4(type):
    """This is the constructior for all the File-like classes.
    Basically it implements auto-registration of subclasses to the File
    class to make File being aware of all its children and have a
    dictionary that maps str_type:subclass.

    This allows function like File.get_all() to give back a list of all
    files in a netshark appliance with the correct subclass
    """
    def __new__(cls, name, bases, attrs):
        file_subclass = super(FileMeta4, cls).__new__(cls, name, bases, attrs)
        for type_str in attrs['type_list']:
            File4.types[type_str] = file_subclass
        return file_subclass

class _AggregatedFile(File4):

    @classmethod
    def _create_aggregated_file(cls, type, shark, path, files):
        """Internal method to create a virtual file.
        This is used by create_merge_file and create_multisegment_file
        """
        error_msg = "An error occurred creating the aggregated file  "
        assert shark is not None
        assert path is not None and len(path) > 0
        assert isinstance(files, list) and len(files) > 0

        if type != "MULTISEGMENT_FILE" and type != "MERGED_FILE":
            raise NetSharkException(error_msg + \
                                 ": unknown aggregated file type " + type)

        aggr_file_dir, aggr_file_name = os.path.split(path)

        #Remove leading and trailing white space from the new resource name
        aggr_file_name = aggr_file_name.strip()

        json_dict = {}
        json_dict["type"] = type
        linked_files = []

        for linked_file in files:
            src_dir, file_name = os.path.split(linked_file.data["id"])
            linked_file_dict = {}
            linked_file_dict["path"] = file_name

            if type == "MULTISEGMENT_FILE":
                linked_file_dict["default_source"] = linked_file.default_file
                linked_file_dict["timeskew"] = linked_file.timeskew_val
            linked_files.append(linked_file_dict)

        json_dict["linked_sources"] = linked_files
        headers = {'Content-Disposition' : aggr_file_name}
        shark.api.fs.upload_xjobject(aggr_file_dir, json_dict, headers)

        new_path = aggr_file_dir + shark._file_separator + aggr_file_name
        if type == "MULTISEGMENT_FILE":

            return MultisegmentFile4(shark, {'id': new_path})
        else:
            return MergedFile4(shark, {'id': new_path})

    def list_linked_files(self):
        """ List the linked files that compose the aggregated file
        """
        trace_file_list = []
        assert self.shark is not None
        assert self.data is not None

        # The path parameter in a multisegment file contains only
        # the file name, therefore we have to add the resource
        # directory before it
        dir_name, file_name = os.path.split(self.data["id"])

        for linked_file in self.data["linked_sources"]:
            linked_file_path = dir_name + self.shark._file_separator + linked_file["path"]
            trace_file_list.append(TraceFile4(self.shark, linked_file_path))

        return trace_file_list

    def _load(self):
        self.data = self.shark.api.fs.get_details(self.data['id'], details=True)


class TraceFile4(File4):
    """This class contains method to manage Trace files.
    """
    __metaclass__ = FileMeta4
    default_file = False
    timeskew_val = 0
    description = ""
    type_list = ['PCAP_FILE', 'PCAPNG_FILE', 'ERF_FILE']

    def __repr__(self):
        assert self.data is not None
        src_dir, file_name = os.path.split(self.data["id"])
        return '<TraceFile4 {0}>'.format(file_name)

    @classmethod
    def upload(cls, shark, path, local_path):
        """Upload a trace file and it returns a reference  to  it.
        'path' contains the remote path, 'local_path' is the local
        file path, 'remote_file_name' contains only the file name
        """
        error_msg = "An error occurred uploading a trace" \
                    "file on the remote directory "
        assert shark is not None
        assert path is not None and len(path) > 0
        cls._validate_local_file(local_path, error_msg)

        if cls.exists(shark, path):
            raise NetSharkException(error_msg + ": A file " +
                                    "at that path already exists")

        file_dir, file_name = os.path.split(path)

        #Remove leading and trailing white space from a string
        file_name = file_name.strip()

        local_file_ref = open(local_path, "rb")

        shark.api.fs.upload_trace(file_dir, file_name, local_file_ref)
        local_file_ref.close()
        new_path = file_dir + shark._file_separator + file_name
        return TraceFile4(shark, {'id':new_path})

    def create_index(self):
        """Create an index on the trace file
        """
        error_msg = "An error occurred creating an index on the trace file "
        assert self.shark is not None
        assert self.data is not None

        try:
            self.shark.api.fs.create_index(self.data["id"])
        except Exception as e:
            msg = error_msg + self.data["id"] + ": " + str(e)
            raise NetSharkException(msg)

        self._update_details(self.data["id"])

    def remove_index(self):
        """Remove the index associated with the trace file
        """
        error_msg = "An error occurred removing an index on the trace file "
        assert self.shark is not None
        assert self.data is not None

        try:
            self.shark.api.fs.delete_index(self.data["id"])
        except Exception as e:
            msg = error_msg + self.data["id"] + ": " + str(e)
            raise NetSharkException(msg)

        self._update_details(self.data["id"])

    def index_info(self):
        """
        Returns index info if the file has an index associated
        """
        error_msg = "An error occurred removing an index on the trace file "
        assert self.shark is not None
        assert self.data is not None

        try:
            result = self.shark.api.fs.index_info(self.data["id"])
        except Exception as e:
            msg = error_msg + self.data["id"] + ": " + str(e)
            raise NetSharkException(msg)

        self._update_details(self.data["id"])
        return result

    def checksum(self):
        """
        Calculates the file checksum
        """
        assert self.shark is not None
        assert self.data is not None

        checksum = self.shark.api.fs.checksum(self.data["id"])

        return checksum

    @classmethod
    def _validate_local_file(cls, local_path, error_msg):
        if local_path == None:
            raise Exception(error_msg + local_path + \
                            ": the local file reference is None")

        if not os.path.isfile(local_path):
            raise Exception(error_msg + local_path + \
                            ": the local file does not exist")

    @property
    def link_layer(self):
        """The file link type (e.g. ethernet, 802.11...), as a lipbap DLT
        (see `<http://www.tcpdump.org/pcap3_man.html>`_)"""
        return self.data["link_type"]

    @property
    def trace_type(self):
        """ The trace file type PCAP_FILE, PCAPNG_FILE, ERF_FILE
        """
        return self.data["type"]


class MultisegmentFile4(_AggregatedFile):
    """This class contains method to manage Multisegment files.
    """
    __metaclass__ = FileMeta4
    type_list = ['MULTISEGMENT_FILE']

    def __repr__(self):
        assert self.data is not None
        src_dir, file_name = os.path.split(self.data["id"])
        return '<MultisegmentFile4 {0}>'.format(file_name)

    @classmethod
    def create_multisegment_file(cls, shark, path, files=None):
        """Creates a multisegment file starting from a
        """
        if files is None:
            files = []
        return cls._create_aggregated_file(cls.type_list[0], shark, path, files)

    def calculate_timeskew(self, packets):
        """
        Start the timeskew computation on the trace file. 'packets' contains
        the number of packets used for the timeskew computation
        """
        error_msg = "An error occurred calculating" \
                    "the time skew on the multisegment file "
        assert self.shark is not None
        assert self.data is not None

        if not isinstance(packets, int):
            raise NetSharkException(error_msg + \
                ": the 'packets' parameter is not an integer value")

        self.shark.api.fs.update_timeskew(self.data["id"], packets)

    def delete_timeskew(self):
        """Delete the timeskew computation on the trace file.
        """
        error_msg = ("An error deleting the timeskew calculation"
                     "on the multisegment file ")
        assert self.shark is not None
        assert self.data is not None

        self.shark.api.fs.delete_timeskew(self.data["id"])

    def timeskew(self):
        """Return timeskew computation details.
        """
        assert self.shark is not None
        assert self.data is not None

        return self.shark.api.fs.get_timeskew(self.data["id"])


class MergedFile4(_AggregatedFile):
    __metaclass__ = FileMeta4
    type_list = ['MERGED_FILE']

    def __repr__(self):
        assert self.data is not None
        src_dir, file_name = os.path.split(self.data["id"])
        return '<MergedFile4 {0}>'.format(file_name)

    @classmethod
    def create_merged_file(cls, shark, path, files=None):
        """Creates a multisegment file starting from a
        """
        if files is None:
            files = []
        return cls._create_aggregated_file(cls.type_list[0], shark, path, files)



#!/usr/bin/env python

# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script prints out the jobs on a given shark.
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.common.datautils import Formatter


class ListJobs(NetSharkApp):
    def add_options(self, parser):
        super(ListJobs, self).add_options(parser)

    def validate_args(self):
        """ Ensure columns are included """
        super(ListJobs, self).validate_args()

    def console(self, header, data):
        print ''
        print header
        print '-' * len(header)

        Formatter.print_table(data, ('id', 'source', 'source_path'))

    def main(self):

        # Get capture jobs
        jobs = self.netshark.get_capture_jobs()
        data = [(x, job.name, job.source_path) for x, job in enumerate(jobs)]
        self.console('Capture Jobs', data)

        clips = self.netshark.get_clips()
        data = [(x, 'Clip: ' + c.description, c.source_path)
                for x, c in enumerate(clips)]
        self.console('Clips', data)

        files = self.netshark.get_files()
        data = [(x, 'File: ' + f.path, f.source_path)
                for x, f in enumerate(files)]
        self.console('Uploaded Files/PCAPs', data)


if __name__ == '__main__':
    ListJobs().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This module contains the NetShark class, which is the main interface to
a SteelCentral NetShark appliance. It allows, among other things, retrieving
the state of the netshark, modifying its settings and performing operations
like creating clips or starting/stopping capture jobs.
"""

from __future__ import absolute_import
import traceback

from steelscript.common.api_helpers import APIVersion
from steelscript.common.service import Service
from steelscript.common.exceptions import RvbdException, RvbdHTTPException
from steelscript.netshark.core._exceptions import NetSharkException
from steelscript.netshark.core._api_helpers import NetSharkAPIVersions
from steelscript.netshark.core._api4 import API4_0
from steelscript.netshark.core._api5 import API5_0
from steelscript.common.datastructures import ColumnProxy
from steelscript.netshark.core._class_mapping import Classesv4, Classes, Classesv5


FILTERS_MAP = {}

API_TABLE = {
    "4.0": API4_0,
    "5.0": API5_0
}

CLASS_TABLE = {
    '4.0': Classesv4,
    '5.0': Classesv5
}


from steelscript.netshark.core.filters import TimeFilter

__all__ = ['NetShark']


class NetShark(Service):
    """The NetShark class is the main interface to interact with a NetShark
    appliance.

    Among other things, it makes it possible to manage views, jobs, files and
    trace clips, and to query and modify the appliance settings.
    """
    def __init__(self, host, port=None, auth=None,
                 force_version=None):
        """Establishes a connection to a NetShark appliance.

        :param str host: the name or IP address of the NetShark to connect to

        :param int port: the TCP port on which the NetShark appliance
            listens.  if this parameter is not specified, the function
            will try to automatically determine the port.

        :param auth: defines the
            authentication method and credentials to use to access the
            NetShark.  It should be an instance of
            :class:`UserAuth <steelscript.common.service.UserAuth>` or
            :class:`OAuth <steelscript.common.service.OAuth>`.

        :param str force_version: is the API version to use when
            communicating.  if unspecified, this will use the latest
            version supported by both this implementation and the
            NetShark appliance.

        See the base :class:`Service <steelscript.common.service.Service>`
        class for more information about additional functionality supported.
        """

        if force_version is not None:
            if not force_version in API_TABLE:
                raise NetSharkException("API version %s unsupported" % force_version)
            versions = [APIVersion(force_version)]
        else:
            versions = [APIVersion(NetSharkAPIVersions.CURRENT)] + \
                       [APIVersion(v) for v in NetSharkAPIVersions.LEGACY]

        super(NetShark, self).__init__("shark", host, port=port, auth=auth,
                                       versions=versions)

        self.api = API_TABLE[str(self.api_version)](self)
        self.classes = CLASS_TABLE[str(self.api_version)]()
        # these module may not be available
        try:
            self.settings = self.classes.Settings(self)
        except NotImplementedError:
            self.settings = Classes()

        # Get the server info
        self.serverinfo = self.get_serverinfo()

        self.views = {}
        self._interfaces = None
        self.xtfields = {}

        def _get_fields():
            self._fetch_extractor_fields()
            return self.xtfields.items()
        def _set_fields(f):
            self.columns = f
        self.columns = ColumnProxy(_get_fields, _set_fields)

        self._capture_jobs_cache = None
        self._traceclips_cache = None
        self._tracefiles_cache = None

    def __repr__(self):
        if self.port is not None:
            return '<NetShark %s:%d>' % (self.host, self.port)
        else:
            return '<NetShark %s>' % (self.host)

    def _get_supported_versions(self):
        try:
            # First try the standard GL7 method
            versions = super(NetShark, self)._get_supported_versions()

            if versions:
                return versions
        except RvbdHTTPException as e:
            if e.status != 404:
                raise

        # older sharks export the protocol info on /protocol_info, and
        # return a 401 unauthorized for any other unrecognized URLs
        res = self.conn.json_request('GET', '/protocol_info')
        if res.status != 200 \
               or res.getheader('content-type') != 'text/xml':
            # any other non-ok status probably means we are
            # not talking to a netshark
            return None

        from xml.etree import ElementTree
        tree = ElementTree.fromstring(res.read())
        if tree.tag != 'ProtocolInfo':
            # probably some non-netshark http server
            return None

        return [APIVersion(tree.get('Version'))]

    def logout(self):
        if self.conn:
            try:
                self.api.common.logout()
            except AttributeError:
                pass

            super(NetShark, self).logout()

    def _detect_auth_methods(self):
        if self.api_version >= APIVersion("4.0"):
            # 4.0 netshark supports the standard GL7 method
            return Service._detect_auth_methods(self)

        self._supports_auth_basic = self.api_version >= APIVersion("3.0")
        self._supports_auth_cookie = self.api_version >= APIVersion("3.1")
        self._supports_auth_oauth = False

    def get_open_views(self):
        """Get a list of View objects, one for each open view on the NetShark appliance.
        """
        self._refresh_views()
        return self.views.values()

    def get_open_view_by_handle(self, handle):
        """Look up the view ``handle`` and return its View object"""
        if handle not in self.views:
            self._refresh_views()

        return self.views[handle]

    def _add_view(self, viewobj):
        """Add ``viewobj`` to the internal cache of view objects """
        # No handle if view hasn't yet been applied
        if not hasattr(viewobj, 'handle'):
            return
        self.views[viewobj.handle] = viewobj

    def _del_view(self, viewobj):
        """Remove ``viewobj`` from the internal cache of view objects """
        try:
            del self.views[viewobj.handle]
        except KeyError:
            pass

    def _refresh_views(self):
        oldviews = self.views
        self.views = {}

        try:
            handles = self.classes.View._get_all(self)
        except:
            # XXX make the except clause narrower
            traceback.print_exc()

            # We don't know if the server is down and we should delete all
            # view objects, or if it's simply momentarily unreachable and
            # we should keep the view objects -- so we take the conservative
            # route and keep them.
            return

        for handle in handles:
            # don't create a new object for the view if we already have one
            if handle in oldviews:
                self.views[handle] = oldviews[handle]
            else:
                try:
                    self.views[handle] = self.classes.View(self, handle)
                except:
                    # XXX make the except clause narrower
                    traceback.print_exc()

                    # XXX should put some sort of "orphaned view"
                    # object here so we know it exists but is unusable
                    pass

    def create_view_from_template(self, source, template,
                                  name=None, sync=True):
        """ Create a new view on this NetShark using `template`.

        For 9.5.x and earlier, `template` should be an XML view
        document, for later releases, `template` should be a
        JSON view description.

        :returns: :class:`View4`

        """
        view = self.classes.View._create_from_template(self, source, template,
                                                       name, sync)
        self._add_view(view)
        return view

    def create_view(self, src, columns, filters=None,
                    start_time=None, end_time=None,
                    name=None, charts=None, sync=True,
                    sampling_time_msec=None):
        """ Create a new view on this NetShark.

        :param src: identifies the source of packets to be analyzed.
            It may be any packet source object.

        :param columns: specifies what information is extracted from
            packets and presented in this view.  It should be a list
            of :py:class:`Key <steelscript.netshark.core.types.Key>`
            and :py:class:`Value <steelscript.netshark.core.types.Value>`
            objects

        :param filters: an optional list of filters that can be used
            to limit which packets from the packet source are
            processed by this view.

        :returns: :class:`View4`

        """

        if start_time is not None or end_time is not None:
            if start_time is None or end_time is None:
                raise ValueError('must specify both start and end times')
            if filters is None:
                filters = []
            filters.append(TimeFilter(start_time, end_time))

        filterobjs = []
        if filters is not None:
            filterobjs.extend([filt.bind(self) for filt in filters])

        view = self.classes.View._create(self, src, columns, filterobjs, name=name,
                                         sync=sync, sampling_time_msec=sampling_time_msec)
        self._add_view(view)
        return view

    def create_job(self, interface, name,
             packet_retention_size_limit,
             packet_retention_packet_limit=None,
             packet_retention_time_limit=None,
             bpf_filter=None,
             snap_length=65525,
             indexing_size_limit=None,
             indexing_synced=False,
             indexing_time_limit=None,
             start_immediately=True,
             requested_start_time=None,
             requested_stop_time=None,
             stop_rule_size_limit=None,
             stop_rule_packet_limit=None,
             stop_rule_time_limit=None):
        """Create a Capture Job on this NetShark, and return it as a
        a capture job object that can be used to work with the new job.

        :param interface: is an interface object identifying the
            source of the packets that this job will capture. The
            method `get_interfaces()` returns interface objects for
            all interfaces on this netshark.

        :param name: is the name that the job will have after being
            created, and that will be used to identify the job for
            successive operations.

        :param packet_retention_size_limit: is the maximum size on
            disk that the job will reach before old packets begin to
            be discarded.  `packet_retention_size_limit` can be
            expressed as a number (in which case the unit will be
            bytes), or as a string.  The format of the string can be
            Number and unit (e.g."1.3GB"),
            or percentage of total disk spoace (e.g. "20%").

        :param packet_retention_packet_limit: the maximum number of
            packets that the job can contain before old packets begin
            to be discarded.

        :param packet_retention_time_limit: a `datetime.timedelta`
            object specifying the maximum time interval that the job
            may contain before old packets begin to be discarded.

        :param bpf_filter: is a filter, based on the Wireshark capture
            filter syntax, that selects which incoming packets should
            be saved in the job.

        :param snap_length: is the portion of each packet that will be
            written on disk, in bytes. The default value of 65535
            ensure that every packet is captured in its entirety.

        :param indexing_size_limit: is the maximum size on disk that
            the index may reach before old index data may be
            discarded.  It is specified in the same format as
            `keep_size`.

        :param indexing_synced: if True the job's microflow index
            will be automatically pruned to cover the same time
            extension of the captured packets.

        :param indexing_time_limit: is a `datetime.timedelta` object
            indicating how long index data should be retained.  This
            argument is only meaningful if `index_and_capture_synced`
            is False.

        :param start_immediately: if True, the Job is started after
            it has been created.

        :param requested_start_time: and `requested_stop_time` are
            `datetime.datetime` objects that, if specified, determine
            the absolute time when the job will start/stop.

        :param stop_rule_size_limit: is specified, the job will stop
            storing new packets when the given size on disk is
            reached.  It is specified in the same format as
            `keep_size`.

        :param stop_rule_packet_limit: if specified, the job will stop
            storing new packets when the given number of packets is
            reached.

        :param stop_rule_time_limit: if specified, it should be a
            `datetime.timedelta` object, and the job will stop storing
            new packets when the given time has elapsed.
        """
        self._capture_jobs_cache = None
        return self.classes.Job.create(self,
                          interface, name, packet_retention_size_limit,
                          packet_retention_packet_limit,
                          packet_retention_time_limit,
                          bpf_filter, snap_length, indexing_size_limit,
                          indexing_synced, indexing_time_limit,
                          start_immediately,
                          requested_start_time, requested_stop_time,
                          stop_rule_size_limit, stop_rule_packet_limit,
                          stop_rule_time_limit)

    def create_clip(self, job, filters, description, locked=True):
        """Create a clip in the NetShark appliance
        """
        self._traceclips_cache = None
        clip = self.classes.Clip.add(self, job, filters, description, locked)
        return clip

    def get_interfaces(self, force_refetch=False):
        """Return a list of Interface objects, corresponding to the
        capture interfaces available on the NetShark.

        If ``force_refetch`` is True, the list of interfaces will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        if force_refetch or self._interfaces is None:
            self._interfaces = self.classes.Interface.get_all(self)
        return self._interfaces

    def get_interface_by_name(self, ifname, force_refetch=False):
        """ Return an Interface object corresponding to the
        interface named ``ifname``.

        If ``force_refetch`` is True, the list of interfaces will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        if force_refetch or self._interfaces is None:
            self._interfaces = self.classes.Interface.get_all(self)

        found = [ ifc for ifc in self._interfaces if ifc.name == ifname ]
        if len(found) == 0:
            raise KeyError()
        assert len(found) == 1
        return found[0]

    def get_capture_jobs(self, force_refetch=False):
        """Return a list of CaptureJob objects, corresponding to the
        capture jobs currently running on the NetShark.

        If ``force_refetch`` is True, the list of jobs will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        if self._capture_jobs_cache is None or force_refetch:
            self._capture_jobs_cache = self.classes.Job.get_all(self)
        return self._capture_jobs_cache

    def get_capture_job_by_name(self, jobname, force_refetch=False):
        """Return a CaptureJob object for the capture job ``jobname``

        If ``force_refetch`` is True, the list of jobs will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        job = None
        if self._capture_jobs_cache is None or force_refetch:
            job = self.classes.Job.get(self, None, name=jobname)
        else:
            for j in self._capture_jobs_cache:
                if j.name == jobname:
                    job = j
                    break
        if job is None:
            raise ValueError('No Job called {0} is available on this NetShark'.format(jobname))
        else:
            return job

    def get_clips(self, force_refetch=False):
        """Return a list of TraceClip objects, corresponding to the
        trace clips available on the NetShark.

        If ``force_refetch`` is True, the list of clips will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        if force_refetch or self._traceclips_cache is None:
            self._traceclips_cache = self.classes.Clip.get_all(self)
        return self._traceclips_cache

    def get_trace_clip_by_description(self, description, force_refetch=False):
        """Return a TraceClip object for the trace clip with the given
        description.

        If ``force_refetch`` is True, the list of jobs will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.

        **Note**: Clips don't have descriptions by default. A description can
        be added to a clip by right-clicking on it in Pilot.
        """
        if not force_refetch or self._traceclips_cache is None:
            # get_clips already set up the cache for us
            clips = self.get_clips(force_refetch)
        else:
            clips = self._traceclips_cache

        clips = [ c for c in clips
                  if c.description == description ]

        if len(clips) == 0:
            raise NetSharkException('cannot find clip %s' % description)

        return clips[0]

    def get_files(self, force_refetch=False):
        """Return a list of TraceFile, MergedFile or Multisegment objects,
        corresponding to the trace files available on the NetShark.

        If ``force_refetch`` is True, the list of files will be
        re-fetched from the NetShark.  Otherwise, the list may be cached.
        """
        if force_refetch or self._tracefiles_cache is None:
            self._tracefiles_cache = self.classes.File.get_all(self)
        return self._tracefiles_cache

    def get_file(self, path):
        """Given a path retrieve the `File` associated with it
        """
        return self.classes.File.get(self, path)

    def _fetch_extractor_fields(self, force=False):
        if not force and len(self.xtfields) > 0:
            return

        for f in self.classes.ExtractorField.get_all(self):
            self.xtfields[f.id] = f

    def get_extractor_fields(self):
        """ Return a list of all extractor fields available on this netshark """
        self._fetch_extractor_fields()
        return sorted(self.xtfields.values())

    def find_extractor_field_by_name(self, fname):
        """ Return a specific extractor field given its name """
        self._fetch_extractor_fields()
        return self.xtfields[fname]

    def search_extractor_fields(self, string):
        """Search through the extractor fields to find the ones that
        match the given string in either the field name or the
        description."""
        self._fetch_extractor_fields()

        string = string.lower()
        return [ f for f in self.xtfields.values()
                 if string in f.name.lower() or string in f.desc.lower() ]

    def get_protocol_version(self):
        """Return the API protocol version used by this netshark."""
        return str(self.api_version)

    def get_logininfo(self):
        """Return a dictionary with the information useful prior to
        logging in to the netshark, including the protocol version,
        banner, and whether or not a login purpose should be
        requested from the user.
        """
        return self.api.common.get_auth_info()

    def get_serverinfo(self):
        '''Get the NetShark appliance overall info.

        :return: a named tuple with the server parameters
        '''
        return self.api.system.get_info()

    def get_stats(self):
        '''Get the NetShark appliance storage info
        '''
        res = dict()
        res['storage'] = self.api.stats.get_storage()
        res['memory'] = self.api.stats.get_memory()
        return res

    def restart(self):
        """Restart the NetShark appliance.
        This will issue a system reboot.
        """
        self.api.system.restart({'type':'SHARK'})

    def restart_probe(self):
        """Restart the NetShark probe.
        This will restart the Pilot server only.
        """
        self.api.system.restart({'type':'PROBE'})

    def ping(self):
        '''Ping the probe.

        :return: True/False depending whether the server is up and running or not
        '''
        return self.api.common.ping(self)

    def create_dir(self, path):
        """
        Create a new directory. It will trigger an exception if the directory
        exist.

        :return: a reference to the new directory
        """
        return self.classes.Directory.create(self, path)

    def get_dir(self, path):
        """
        Get a directory. It will trigger an exception if the directory does not
        exist.

        :return: a reference to the directory
        """
        return self.classes.Directory(self, path)

    def exists(self, path):
        """
        Check if a path exists, works for files or directories.

        :return: true if the path exists, false otherwise
        """
        return self.classes.Directory.exists(self, path)

    def upload_trace_file(self, path, local_file):
        """
        Uploads a new trace file. 'path' is the complete destination path,
        'local_file' is the the local file to upload.

        :return: reference to the new trace file
        """
        return self.classes.TraceFile.upload(self, path, local_file)

    def create_multisegment_file(self, path, files=None):
        """
        Creates a multisegment file. 'path' is the new file full name and
        'files' is a File objects list

        :return: a reference to the new file
        """
        return self.classes.MultisegmentFile.create_multisegment_file(self, path, files)

    def create_merged_file(self, path, files=None):
        """
        Creates a merged file. 'path' is the new file full name and
        'files' is a File objects list

        :return: a reference to the new file
        """
        return self.classes.MergedFile.create_merged_file(self, path, files)

    def download_log(self, path=None, log_type='COMPLETE', case_id=None):
        """Download log archive to local machine into path
        If path is None a temporary file is created

        `log_type` can be:

        * 'CURRENT'
        * 'PROBE'
        * 'PACKETRECORDER'
        * 'COMPLETE' (default)

        `case_id` is an integer that represent the case id
        """
        config = {'dump_type': log_type}
        if case_id:
            config['case_id'] = int(case_id)
        return self.api.system.get_sysdump(path, config)

    def create_export(self, source, timefilter, filters=None,
                      wait_for_data=False, wait_duration=10):
        """Create a pcap Export object.

        :param source: source to use for packets: Job, View, Interface
            or TraceClip

        :param timefilter: time range of packets to export

        :param filters: additional filters

        :param wait_for_data: Boolean to indicate if additional retries
            should be made if the export request initially fails.  If True,
            up to three attempts will be made with an interval of
            ``wait_duration``.

        :param wait_duration: interval of time to wait between export
            attempts, if ``wait_for_data`` is True.

        :returns: :class:`Export4`

        """
        return self.classes.Export.create(self, source, timefilter, filters,
                                          wait_for_data, wait_duration)

    @property
    def version(self):
        """Returns the NetShark software version
        """
        return self.api.common.info().get('sw_version')

    @property
    def model(self):
        """Returns the NetShark software model
        """
        return self.api.common.info().get('model')

    @property
    def _file_separator(self):
        # Get the file separator based on the server OS
        if self.serverinfo['system_type'] == "Windows":
            return "\\"
        else:
            return "/"



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import functools
import logging
import time

from steelscript.common.datastructures import DictObject
from steelscript.common.timeutils import datetime_to_seconds
from steelscript.common.exceptions import RvbdHTTPException

logger = logging.getLogger(__name__)


class NetSharkExportException(Exception):
    pass


def loaded(f):
    @functools.wraps(f)
    def wrapper(self, *args, **kwds):
        self._ensure_loaded()
        return f(self, *args, **kwds)
    return wrapper


class _InputSource(object):
    def __init__(self, shark, data, api):
        self.shark = shark
        self.id = data['id']
        self.data = data
        self._api = api

    @classmethod
    def get(cls, shark, id, name=None):
        for source in cls.get_all(shark):
            if (id and (source.id == id) or
                    name and (source.data['config']['name'] == name)):
                return source
        return None

    def update(self, data):
        """Update the data of the current object
        with new data from the server
        """
        assert self.id == data['id']
        self.data = DictObject.create_from_dict(data)


class Clip(_InputSource):

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.delete()

    def is_live(self):
        """Can be used to determine if a source object is live or offline.
        The result is always False for a TraceClip."""
        return False

class View(object):
    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    @classmethod
    def _get_all(cls, shark):
        return shark.api.view.get_all()

    def _ensure_output(self):
        if len(self._outputs) == 0:
            # change this since postapply doesn't load
            # the view parameters
            self._postapply()

    @classmethod
    def _process_view(cls, view, source, sync):
        if sync:
            try:
                if not source.is_live():
                    view._poll_completion()
                view._postapply()
            except:
                view.close()
                raise

        return view

    def get_iterdata(self, *args, **kwargs):
        """ Returns an iterator for the data from the output
        in this view.  Shorthand for `all_outputs()[0].get_iterdata()`.

        Raises a LookupError if the view has more than one output.

        For a full description of the function arguments, refer to the
        method Output.get_iterdata()
        """
        outputs = self.all_outputs()
        if len(outputs) != 1:
            raise LookupError('This view has more than one output. You have to call get_iterdata'
                              'on an Output object directly')
        return outputs[0].get_iterdata(*args, **kwargs)

    def get_data(self, *args, **kwargs):
        """ Returns the data from the output in this view.
        Shorthand for `all_outputs()[0].get_data()`.

        Raises a LookupError if the view has more than one output.

        For a full description of the function arguments, refer to the
        method Output.get_data().
        """
        outputs = self.all_outputs()
        if len(outputs) != 1:
            raise LookupError('This view has more than one output. You have to call get_data'
                              'on an Output object directly')

        return outputs[0].get_data(*args, **kwargs)

    def get_legend(self):
        """ Returns the legend from the output in this view.
        Shorthand for `all_outputs()[0].get_legend()`.

        Raises a LookupError if the view has more than one output.
        """
        outputs = self.all_outputs()
        if len(outputs) != 1:
            raise LookupError('This view has more than one output. You have to call get_legend'
                              'on an Output object directly')

        return outputs[0].get_legend()

    def all_outputs(self):
        """ Return a list of Output objects, one for each output
        in this view """
        self._ensure_output()
        return self._outputs.values()

    def get_output(self, id):
        """ Return the Output object corresponding to
        the output ``id``.
        """
        self._ensure_output()
        return self._outputs[id]

class Output(object):
    def get_data(self, start=None, end=None, delta=None,
                 aggregated=False,
                 sortby=None, sorttype="descending",
                 fromentry=0, toentry=0,):
        """
        Get the data for this view. This function downloads the whole
        dataset before returning it, so it's useful when random access to the
        view data is necessary.

        The arguments have the same meanings as corresponding arguments
        to get_iterdata(), see its documentation for a full explanation
        of all arguments and their meanings.
        """
        it = self.get_iterdata(start, end, delta, aggregated,
                               sortby, sorttype, fromentry, toentry)
        data = list()
        for row in it:
            data.append(row)
        return data

class File(_InputSource):
    """A file packet source. A file can be in one of the several
       formats that NetShark supports, e.g. pcap, pcapNG or ERF. Files are
       usually created using the "send to file" feature in Pilot, but they can
       also be uploaded into the appliance using the netshark.upload() method.
       TraceClip objects are normally not instantianted directly, but are
       instead obtained by calling
       :py:func:`steelscript.netshark.core.netshark.NetShark.get_files` or
       :py:func:`CaptureJob.get_file`."""

    def is_live(self):
        """Can be used to determine if a source object is live or offline.
        The result is always False for a File."""
        return False


class Job(_InputSource):
    def __enter__(self):
        return self

    def __exit__(self, _type, value, traceback):
        self.delete()


class Export(object):
    """A pcap export object.

    Export objects are not instantiated directly, but are instead
    obtained by calling
    :py:func:`steelscript.netshark.core.netshark.NetShark.create_export`

    """

    def __init__(self, shark, id, source):
        self.shark = shark
        self.id = id
        self.source = source

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.delete()

    @classmethod
    def create(cls, shark, source, timefilter, filters=None,
               wait_for_data=False, wait_duration=10):
        """Create a new pcap export from the given source."""
        config = {
            'output_format': 'PCAP_US',
            'start_time': datetime_to_seconds(timefilter.start),
            'end_time': datetime_to_seconds(timefilter.end)
            }

        if filters:
            config['filters'] = [filt.bind(shark) for filt in filters]

        r = None
        cnt = 0
        while r is None and cnt < 3:
            try:
                r = source._api.create_export(source.id, config=config)
            except RvbdHTTPException, e:
                if 'job is empty' in str(e) and wait_for_data:
                    logger.warning("Data is not available to export,"
                                   " waiting for %s seconds" % wait_duration)
                    time.sleep(wait_duration)
                    cnt += 1
                    continue
                else:
                    raise NetSharkExportException(str(e))

        if r is None:
            raise NetSharkExportException("Data is not available to export,"
                                          " need to wait longer or "
                                          " ensure the job id is correct")

        return cls(shark, r['id'], source)

    def details(self):
        """Return details about an export."""
        return self.source._api.get_export_details(self.source.id, self.id)

    def download(self, filename, overwrite=False):
        """Download the packets as a pcapfile to a local file."""
        if self.details()['status']['state'] != 'RUNNING':
            return False

        self.source._api.get_packets_from_export(self.source.id, self.id,
                                                 filename, overwrite=overwrite)
        return True

    def delete(self):
        """Delete the export on the shark.

        Note that downloading an export automatically deletes the export.
        """
        try:
            self.source._api.delete_export(self.source.id, self.id)
        except:
            # Export probably no longer exists
            pass



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



from steelscript.netshark.core import *
from steelscript.netshark.core.app import NetSharkApp
import time
import logging

logging.basicConfig(level=logging.DEBUG,
                    format="%(asctime)s [%(levelname)-5.5s] %(msg)s")

def test(app):
    t = 60
    while True:
        info = app.shark.get_serverinfo()
        print "%s Success... (sleeping for %s seconds)" % (time.ctime(time.time()), t)
        time.sleep(t)
        t = t * 2

app = NetSharkApp(main_fn = test)
app.run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import steelscript.netshark.core
from steelscript.netshark.core import NetShark
from steelscript.netshark.core.types import Operation, Value, Key
from steelscript.netshark.core.filters import NetSharkFilter, TimeFilter
from steelscript.common.service import UserAuth

from testconfig import config

import os
import sys
import time
import logging

#these tests are not compatible with nosetests
if sys.argv[0].find('nosetests') != -1:
    raise Exception("These tests are not compatible with nosetest loader, "
                    "nosetests does not support with testscenarios module, "
                    "for more information: https://bugs.launchpad.net/testscenarios/+bug/872887")

http_loglevel = 0
debug_msg_body = 0
scenario_only = False


logger = logging.getLogger(__name__)

loglevel = config.get('loglevel')
logging.basicConfig(format="%(asctime)s [%(levelname)-5.5s] %(msg)s",
                    level=loglevel or logging.DEBUG)

import steelscript.common.connection
try:
    steelscript.common.connection.Connection.HTTPLIB_DEBUGLEVEL = config['http_loglevel']
except KeyError:
    pass

try:
    steelscript.common.connection.Connection.DEBUG_MSG_BODY = config['debug_http_body']
except KeyError:
    pass


HERE = os.path.abspath(os.path.dirname(__file__))
trace_files_dir = os.path.join(HERE, "traces")


def create_shark(host, version):

    username = 'admin'
    password = 'admin'
    auth = UserAuth(username, password)
    sk = NetShark(host, auth=auth, force_version=version)

    return sk


def setup_defaults():
    #
    # some columns and filters we can use for creating views
    #
    columns = [Key('ip.src'),
               Key('ip.dst'),
               Value('generic.packets'),
               Value('http.duration', Operation.max, description="Max Duration"),
               Value('http.duration', Operation.avg, description="Avg Duration")]
    # we don't
    # have generic.application in 5.0 anymore
    filters = [NetSharkFilter('(tcp.src_port=80) | (tcp.dst_port=80)'),
               TimeFilter.parse_range('last 2 hours')]
    return columns, filters


def setup_capture_job(shark):
    def create_job():
        interface = shark.get_interfaces()[0]
        if shark.model == 'vShark':
            job = shark.create_job(interface, 'Flyscript-tests-job', '10%',
                                   indexing_size_limit='2GB', start_immediately=True)
        else:
            job = shark.create_job(interface, 'Flyscript-tests-job', '400MB',
                                   indexing_size_limit='300MB', start_immediately=True)
            time.sleep(5)
            job.stop()
        return job

    try:
        job = shark.get_capture_job_by_name('Flyscript-tests-job')
    except ValueError:
        #let's create a capture job
        job = create_job()

    if job.size_on_disk == 0:
        #job has no packets, probably
        #the packet storage has been formatted
        #and lost all the packages
        job.delete()
        job = create_job()

    logger.info('using capture job %r' % job)
    return job


def create_trace_clip(shark, job):
    # create a relatively short trace clip that we can use later
    fltr = TimeFilter.parse_range('last 10 minutes')
    clip = shark.create_clip(job, [fltr], 'test_clip')
    logger.info('created test trace clip')
    return clip


def create_tracefile(shark):
    try:
        tracefile = shark.get_file('/admin/test.pcap')
    except steelscript.netshark.core.RvbdHTTPException as e:
        if e.error_text.find('does not exist') == -1:
            raise

        dir = shark.get_dir(shark.auth.username)
        local_file = os.path.join(trace_files_dir, "2-router1-in.pcap")
        tracefile = dir.upload_trace_file("test.pcap", local_file)
        logger.info('uploaded test trace file')
    return tracefile


def cleanup_shark(shark):
    """Does proper cleanup of the netshark appliance from views, jobs and clips

    In each case, only items named with a prefix of 'test_' are removed.
    """
    # XXX investigate implementing this at end of all tests instead of
    #     after every test

    for v in shark.api.view.get_all():
        config = shark.api.view.get_config(v['id'])
        if 'info' in config and config['info']['title'].startswith('test_'):
            shark.api.view.close(v['id'])

    for j in shark.api.jobs.get_all():
        if j['config']['name'].startswith('test_'):
            shark.api.jobs.delete(j['id'])

    for c in shark.api.clips.get_all():
        if c['config']['description'].startswith('test_'):
            shark.api.clips.delete(c['id'])


class SetUpTearDownMixin(object):
    """Used to store the setUp and tearDown function used by
    the test classes"""
    def setUp(self):
        self.shark = create_shark(self.host, self.version)

    def tearDown(self):
        cleanup_shark(self.shark)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import time
import pandas
import logging
import hashlib
import threading

from django import forms

from steelscript.netshark.core.types import Operation, Value, Key
from steelscript.netshark.core.filters import NetSharkFilter, TimeFilter, \
    BpfFilter
from steelscript.netshark.core._class_mapping import path_to_class
from steelscript.netshark.appfwk.models import NetSharkViews

from steelscript.common.timeutils import (parse_timedelta,
                                          timedelta_total_seconds,
                                          datetime_to_nanoseconds,
                                          datetime_to_microseconds,
                                          nsec_string_to_datetime)

from steelscript.appfwk.apps.devices.devicemanager import DeviceManager
from steelscript.appfwk.apps.devices.forms import fields_add_device_selection
from steelscript.appfwk.apps.datasource.models import \
    Column, TableField, DatasourceTable, TableQueryBase
from steelscript.appfwk.apps.datasource.forms import \
    fields_add_time_selection, fields_add_resolution
from steelscript.appfwk.libs.fields import Function
from steelscript.appfwk.apps.datasource.forms import IDChoiceField
from steelscript.appfwk.apps.jobs import QueryComplete
from steelscript.appfwk.apps.devices.models import Device

logger = logging.getLogger(__name__)
lock = threading.Lock()


def setup_capture_job(netshark, name, size=None):
    """ Reference function to initialize new capture job. """
    if size is None:
        size = '10%'

    try:
        job = netshark.get_capture_job_by_name(name)
    except ValueError:
        # create a capture job on the first available interface
        interface = netshark.get_interfaces()[0]
        job = netshark.create_job(interface, name, size,
                                  indexing_size_limit='2GB',
                                  start_immediately=True)
    return job


def netshark_source_name_choices(form, id_, field_kwargs, params):
    """ Query netshark for available capture jobs / trace clips. """
    netshark_device = form.get_field_value('netshark_device', id_)
    if netshark_device == '':
        choices = [('', '<No netshark device>')]
    else:
        netshark = DeviceManager.get_device(netshark_device)

        choices = []

        for job in netshark.get_capture_jobs():
            choices.append((job.source_path, job.name))

        for clip in netshark.get_clips():
            choices.append((clip.source_path, 'Clip: ' + clip.description))

        if params['include_files']:
            for f in netshark.get_files():
                choices.append((f.source_path, 'File: ' + f.path))

        if params['include_interfaces']:
            for iface in netshark.get_interfaces():
                choices.append((iface.source_path, 'If: ' + iface.description))

    field_kwargs['label'] = 'Source'
    field_kwargs['choices'] = choices


class NetSharkColumn(Column):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    COLUMN_OPTIONS = {'extractor': None,
                      'operation': None,
                      'default_value': None}


def fields_add_filterexpr(table, keyword='netshark_filterexpr',
                          initial=None):
    field = TableField(keyword=keyword,
                       label='NetShark Filter Expression',
                       help_text='Traffic expression using '
                                 'NetShark filter syntax',
                       initial=initial,
                       required=False)
    field.save()
    table.fields.add(field)


def fields_add_bpf_filterexpr(table, keyword='netshark_bpf_filterexpr',
                              initial=None):
    field = TableField(keyword=keyword,
                       label='NetShark BPF Filter Expression',
                       help_text='Traffic expression using '
                                 'BPF filter syntax',
                       initial=initial,
                       required=False)
    field.save()
    table.fields.add(field)


class NetSharkTable(DatasourceTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _column_class = 'NetSharkColumn'
    _query_class = 'NetSharkQuery'

    TABLE_OPTIONS = {'aggregated': False,
                     'include_files': False,
                     'include_interfaces': False,
                     'include_persistent': False,
                     'include_filter': True,       # included by default
                     'include_bpf_filter': False,
                     }

    FIELD_OPTIONS = {'duration': '1m',
                     'durations': ('1m', '15m'),
                     'resolution': '1s',
                     'resolutions': ('1s', '1m'),
                     }

    def post_process_table(self, field_options):
        duration = field_options['duration']
        if isinstance(duration, int):
            duration = "%dm" % duration

        fields_add_device_selection(self, keyword='netshark_device',
                                    label='NetShark', module='netshark',
                                    enabled=True)

        func = Function(netshark_source_name_choices,
                        self.options)
        TableField.create(
            keyword='netshark_source_name', label='Source',
            obj=self,
            field_cls=IDChoiceField,
            field_kwargs={'widget_attrs': {'class': 'form-control'}},
            parent_keywords=['netshark_device'],
            dynamic=True,
            pre_process_func=func
        )

        if self.options.include_persistent:
            TableField.create(keyword='netshark_persistent',
                              label='Persistent View', obj=self,
                              field_cls=forms.BooleanField,
                              initial=False)

        fields_add_time_selection(self,
                                  initial_duration=duration,
                                  durations=field_options['durations'])
        fields_add_resolution(self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'])

        if self.options.include_filter:
            fields_add_filterexpr(self)

        if self.options.include_bpf_filter:
            fields_add_bpf_filterexpr(self)


class NetSharkQuery(TableQueryBase):

    def run(self):
        """ Main execution method
        """
        criteria = self.job.criteria

        self.timeseries = False       # if key column called 'time' is created
        self.column_names = []

        # Resolution comes in as a time_delta
        resolution = timedelta_total_seconds(criteria.resolution)

        default_delta = 1000000000                      # one second
        self.delta = int(default_delta * resolution)    # sample size interval

        if criteria.netshark_device == '':
            logger.debug('%s: No netshark device selected' % self.table)
            self.job.mark_error("No NetShark Device Selected")
            return False

        shark = DeviceManager.get_device(criteria.netshark_device)

        logger.debug("Creating columns for NetShark table %d" % self.table.id)

        # Create Key/Value Columns
        columns = []
        for tc in self.table.get_columns(synthetic=False):
            tc_options = tc.options
            if (tc.iskey and tc.name == 'time' and
                    tc_options.extractor == 'sample_time'):
                # don't create column, use the sample time for timeseries
                self.timeseries = True
                self.column_names.append('time')
                continue
            elif tc.iskey:
                c = Key(tc_options.extractor,
                        description=tc.label,
                        default_value=tc_options.default_value)
            else:
                if tc_options.operation:
                    try:
                        operation = getattr(Operation, tc_options.operation)
                    except AttributeError:
                        operation = Operation.sum
                        print ('ERROR: Unknown operation attribute '
                               '%s for column %s.' %
                               (tc_options.operation, tc.name))
                else:
                    operation = Operation.none

                c = Value(tc_options.extractor,
                          operation,
                          description=tc.label,
                          default_value=tc_options.default_value)

            self.column_names.append(tc.name)
            columns.append(c)

        # Identify Sort Column
        sortidx = None
        if self.table.sortcols is not None:
            sortcol = Column.objects.get(table=self.table,
                                         name=self.table.sortcols[0])
            sort_name = sortcol.options.extractor
            for i, c in enumerate(columns):
                if c.field == sort_name:
                    sortidx = i
                    break

        # Initialize filters
        criteria = self.job.criteria

        filters = []

        if hasattr(criteria, 'netshark_filterexpr'):
            logger.debug('calculating netshark filter expression ...')
            filterexpr = self.job.combine_filterexprs(
                exprs=criteria.netshark_filterexpr,
                joinstr="&"
            )
            if filterexpr:
                logger.debug('applying netshark filter expression: %s'
                             % filterexpr)
                filters.append(NetSharkFilter(filterexpr))

        if hasattr(criteria, 'netshark_bpf_filterexpr'):
            # TODO evaluate how to combine multiple BPF filters
            # this will just apply one at a time
            filterexpr = criteria.netshark_bpf_filterexpr
            logger.debug('applying netshark BPF filter expression: %s'
                         % filterexpr)
            filters.append(BpfFilter(filterexpr))

        resolution = criteria.resolution
        if resolution.seconds == 1:
            sampling_time_msec = 1000
        elif resolution.microseconds == 1000:
            sampling_time_msec = 1
            if criteria.duration > parse_timedelta('1s'):
                msg = ("Cannot run a millisecond report with a duration "
                       "longer than 1 second")
                raise ValueError(msg)
        else:
            sampling_time_msec = 1000

        # Get source type from options
        logger.debug("NetShark Source: %s" %
                     self.job.criteria.netshark_source_name)

        source = path_to_class(
            shark, self.job.criteria.netshark_source_name)
        live = source.is_live()
        persistent = criteria.get('netshark_persistent', False)

        if live and not persistent:
            raise ValueError("Live views must be run with persistent set")

        view = None
        if persistent:
            # First, see a view by this title already exists
            # Title is the table name plus a criteria hash including
            # all criteria *except* the timeframe
            h = hashlib.md5()
            h.update('.'.join([c.name for c in
                               self.table.get_columns()]))
            for k, v in criteria.iteritems():
                if criteria.is_timeframe_key(k):
                    continue
                h.update('%s:%s' % (k, v))

            title = '/'.join(['steelscript-appfwk', str(self.table.id),
                              self.table.namespace, self.table.name,
                              h.hexdigest()])
            view = NetSharkViews.find_by_name(shark, title)
            logger.debug("Persistent view title: %s" % title)
        else:
            # Only assign a title for persistent views
            title = None

        timefilter = TimeFilter(start=criteria.starttime, end=criteria.endtime)

        if not view:
            # Not persistent, or not yet created...

            if not live:
                # Cannot attach time filter to a live view,
                # it will be added later at get_data() time
                if criteria.starttime and criteria.endtime:
                    filters.append(timefilter)

                    logger.info("Setting netshark table %d timeframe to %s" %
                                (self.table.id, str(timefilter)))
                else:
                    # if times are set to zero, don't add to filter
                    # this will process entire timeframe of source instead
                    logger.info("Not setting netshark table %d timeframe" %
                                self.table.id)

            # Create it
            with lock:
                logger.debug("%s: Creating view for table %s" %
                             (str(self), str(self.table)))
                view = shark.create_view(
                    source, columns, filters=filters, sync=False,
                    name=title, sampling_time_msec=sampling_time_msec)

            if not live:
                done = False
                logger.debug("Waiting for netshark table %d to complete" %
                             self.table.id)
                while not done:
                    time.sleep(0.5)
                    with lock:
                        s = view.get_progress()
                        self.job.mark_progress(s)
                        self.job.save()
                        done = view.is_ready()

        logger.debug("Retrieving data for timeframe: %s" % timefilter)

        # Retrieve the data
        with lock:
            getdata_kwargs = {}
            if sortidx:
                getdata_kwargs['sortby'] = sortidx

            if self.table.options.aggregated:
                getdata_kwargs['aggregated'] = self.table.options.aggregated
            else:
                getdata_kwargs['delta'] = self.delta

            if live:
                # For live views, attach the time frame to the get_data()
                getdata_kwargs['start'] = (
                    datetime_to_nanoseconds(criteria.starttime))
                getdata_kwargs['end'] = (
                    datetime_to_nanoseconds(criteria.endtime))

            self.data = view.get_data(**getdata_kwargs)

            if not persistent:
                view.close()

        if self.table.rows > 0:
            self.data = self.data[:self.table.rows]

        self.parse_data()

        logger.info("NetShark Report %s returned %s rows" %
                    (self.job, len(self.data)))

        return QueryComplete(self.data)

    def parse_data(self):
        """Reformat netshark data results to be uniform tabular format."""
        out = []
        if self.timeseries:
            # use sample times for each row
            for d in self.data:
                if d['t'] is not None:
                    t = datetime_to_microseconds(d['t']) / float(10 ** 6)
                    out.extend([t] + x for x in d['vals'])

        else:
            for d in self.data:
                out.extend(x for x in d['vals'])

        self.data = out


class NetSharkJobsTable(DatasourceTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _query_class = 'NetSharkJobsQuery'


class NetSharkJobsQuery(TableQueryBase):

    # BPF filter will be rendered wholly if with length less than MAX_LENGTH,
    # Otherwise only the first 17 characters will be shown.
    MAX_LENGTH = 20

    def run(self):

        sks = Device.objects.filter(enabled=True, module='netshark')

        res = []
        for sk in sks:
            sk_dev = DeviceManager.get_device(sk.id)
            for job in sk_dev.get_capture_jobs():

                if_name = job.data['config']['interface_name']

                start = str(nsec_string_to_datetime(job.packet_start_time))
                end = str(nsec_string_to_datetime(job.packet_end_time))

                bpf_filter = job.data['config'].get('bpf_filter', '')

                if len(bpf_filter) > self.MAX_LENGTH:
                    bpf_filter = bpf_filter[:self.MAX_LENGTH - 2] + '...'

                pkts_dropped = job.get_stats()['packets_dropped']
                pkts_written = job.get_stats()['packets_written']

                job_data = dict(netshark=sk.name,
                                job_id=job.data['id'],
                                job_name=job.data['config']['name'],
                                interface=if_name,
                                state=job.data['status']['state'],
                                size=job.data['status']['packet_size'],
                                start_time=start,
                                end_time=end,
                                bpf_filter=bpf_filter,
                                dpi_enabled=str(job.dpi_enabled),
                                index_enabled=str(job.index_enabled),
                                last_sec_dropped=pkts_dropped['last_second'],
                                last_min_dropped=pkts_dropped['last_minute'],
                                last_hr_dropped=pkts_dropped['last_hour'],
                                last_sec_written=pkts_written['last_second'],
                                last_min_written=pkts_written['last_minute'],
                                last_hr_written=pkts_written['last_hour']
                                )
                res.append(job_data)

        return QueryComplete(pandas.DataFrame(res))



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import steelscript.appfwk.apps.report.modules.c3 as c3
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.tables as tables

from steelscript.netshark.appfwk.datasources.netshark import NetSharkTable
#
# Define a NetShark Report and Table
#

report = Report.create('NetShark DNS', position=10)

report.add_section()


# DNS Queries Over time
name = 'DNS Queries and Response Time Over Time'
s = NetSharkTable.create(name, duration=15,
                         resolution='1min', aggregated=False)

s.add_column('time', label='Time', iskey=True, datatype='time',
             extractor='sample_time')
s.add_column('dns_count', label='DNS Query Count', datatype='integer',
             extractor='dns.query.count', operation='sum')
s.add_column('dns_response_time', label='DNS Response Time (ns)',
             units='ms', extractor='dns.response_time', operation='none')
report.add_widget(c3.TimeSeriesWidget, s, name, width=12,
                  altaxis=['dns_response_time'])

# DNS Response Code List for NetShark 1
name = 'DNS Response Codes'
s = NetSharkTable.create(name, duration=15, aggregated=True)

s.add_column('dns_is_success_str', label='DNS Success', iskey=True,
             datatype='string', extractor='dns.is_success_str',
             operation='none')
s.add_column('dns_total_queries', label='DNS Total Queries',
             datatype='integer', extractor='dns.query.count',
             operation='sum', sortdesc=True)

report.add_widget(c3.PieWidget, s, name, width=6)

# DNS Query Type for NetShark 1
name = 'DNS Query Type'
s = NetSharkTable.create(name, duration=15, aggregated=True)

s.add_column('dns_query_type', label='DNS Query Type', iskey=True,
             datatype='string', extractor='dns.query.type', operation='none')
s.add_column('dns_total_queries', label='DNS Total Queries',
             datatype='integer', extractor='dns.query.count',
             operation='sum', sortdesc=True)

report.add_widget(c3.PieWidget, s, name, width=6)

# DNS Request Details Table for NetShark 1
name = 'Top 100 DNS Requests'
s = NetSharkTable.create(name, duration=15, aggregated=True, rows=100)

s.add_column('dns_query_name', label='DNS Request', iskey=True,
             datatype='string', extractor='dns.query.name')
s.add_column('dns_is_success_str', label='Query Result', iskey=True,
             datatype='string', extractor='dns.is_success_str',
             operation='none')
s.add_column('dns_query_type', label='# Requests',
             datatype='string', extractor='dns.query.count',
             operation='sum', sortdesc=True)
report.add_widget(tables.TableWidget, s, name, width=12)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This script shows a couple of advanced options that can be used when creating
Capture Jobs.
"""

import datetime

from steelscript.netshark.core.app import NetSharkApp


class JobCreator(NetSharkApp):
    def main(self):
        # For the sake of simplicity, we pick the first available port on the NetShark
        interface = self.netshark.get_interfaces()[0]

        # This creates a job that captures only web traffic (port 80), and stores
        # on disk only the first 100 bytes of each packet.
        # Capture Job filters are based on the Wireshatk capture filter syntax, a
        # reference of which can be found here:
        #   http://wiki.wireshark.org/CaptureFilters
        name = "Test Job 1"
        job = self.netshark.create_job(interface,
                                       name,
                                       "257MB",
                                       bpf_filter="port 80",
                                       snap_length=100)
        print 'Job named %s created successfully ...' % name,
        job.delete()
        print 'and deleted.'

        # This creates a job that has no Microflow Index.
        # Avoiding indexing the job can be beneficial if performance is very
        # important and drill-down efficiency can be sacrificed.
        name = "Test Job 2"
        job = self.netshark.create_job(interface,
                                       name,
                                       "257MB")
        print 'Job named %s created successfully ...' % name,
        job.delete()
        print 'and deleted.'

        # This creates a job in which packets and index have different retention
        # times.  The index has a much smaller disk footprint compared to packets
        # (usually around 1%), and therefore can cover longer timespans.
        # In this case, packets storage is 500MB or 1 day (whichever is smaller),
        # while index storage is 20MB or one week.
        name = "Test Job 3"
        job = self.netshark.create_job(
            interface,
            name,
            "257MB",
            packet_retention_time_limit=datetime.timedelta(days=1),
            indexing_size_limit="25MB",
            indexing_time_limit=datetime.timedelta(days=7)
        )
        print 'Job named %s created successfully ...' % name,
        job.delete()
        print 'and deleted.'

        print 'Capture Jobs created (and deleted) successfully'


if __name__ == '__main__':
    JobCreator().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
The NetShark package offers a set of interfaces to control and work with
a SteelCentral NetShark appliance.
The functionality in the module includes:

"""

from __future__ import absolute_import


from steelscript.netshark.core.netshark import *
from steelscript.common.exceptions import *
from steelscript.common.service import *
from steelscript.netshark.core._exceptions import *
from steelscript.netshark.core.filters import *
from steelscript.netshark.core.types import *
from steelscript.netshark.core._source4 import *
from steelscript.netshark.core._source5 import *
from steelscript.netshark.core._view4 import *



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script contains several examples that show more advanced view creation
and data retrieval scenarios.
"""

import time
import datetime

from steelscript.netshark.core.app import NetSharkApp
from steelscript.netshark.core.types import Value, Key
from steelscript.netshark.core.filters import NetSharkFilter
from steelscript.netshark.core.viewutils import write_csv

###############################################################################
# Script setup
###############################################################################

CSV_FILE_NAME = "result.csv"


class CreateViewEXApp(NetSharkApp):
    def main(self):
        # Open the remote file
        source = self.netshark.get_file('/admin/noon.cap')

        ######################################################################
        # Applying a view asynchronously.
        # When the amount of packets processed by a view on a netshark
        # appliance is big, it could take minutes for the view data to be
        # generated. This example shows how to use the sync parameter of
        # create_view() to create a view asynchronously and inform the user
        # about the processing progress.
        ######################################################################

        # Specify the column list
        columns = [
            Key(self.netshark.columns.ip.src),
            Value(self.netshark.columns.generic.bytes)
        ]

        # Create the view, making sure that sync is set to False
        v = self.netshark.create_view(source, columns, sync=False)

        # Loop until the progress reaches 100%
        while True:
            p = v.get_progress()
            print p

            if p == 100:
                break

            time.sleep(.5)

        # Retrieve the view data.
        output = v.get_data(aggregated=True)

        # Print the data to the screen
        for sample in output:
            print sample["t"].strftime("%H:%M:%S")
            for vals in sample['vals']:
                print '\t{0}\t{1}'.format(vals[0], vals[1])

        v.close()

        ######################################################################
        # Using filters.
        # Compared to the example in create_view.py, here we add a port 80
        # filter to the view to create a "web top talkers".
        ######################################################################

        # Specify a NetShark filter for port 80 packets. The NetSharkFilter
        # class implements Pilot filters, which means that you can paste any
        # filter from Pilot to the line below. Other valid filter types are
        # WiresharkDisplayFilter (which supports any Wireshark display
        # filter), BpfFilter (for Wireshark capture filters) and TimeFilter
        # (for time-based filtering).
        filters = [NetSharkFilter('tcp.port_pair="80"')]

        # Specify the column list
        columns = [
            Key(self.netshark.columns.ip.src),
            Value(self.netshark.columns.generic.bytes),
        ]

        # The list of filters is passes as a parameter to create_view()
        v = self.netshark.create_view(source, columns, filters)

        # Retrieve the view data.
        output = v.get_data(aggregated=True)

        # Save the data to disk
        write_csv(CSV_FILE_NAME, v.get_legend(), output)

        v.close()

        ######################################################################
        # Time-based data and manual output parsing.
        # This view shows the number of http requests over time, in a format
        # suitable to be represented on a stripchart.
        ######################################################################

        # Specify the column list
        columns = [Value(self.netshark.columns.http.answered_requests)]

        # The list of filters is passes as a parameter to create_view()
        print source, columns
        v = self.netshark.create_view(source, columns)

        # Retrieve the view data.
        # View.get_iterdata() returns an iterator to the view output instead
        # of the full data. This is ideal when the data is very big, because
        # it makes it possible to process it while it's downloaded, saving
        # memory at the client side.
        # Note also that we don't specify aggregated=True. This means that
        # we will receive a sample for each second interval.
        output = v.get_iterdata()

        # Instad of using write_csv(), this time we manually parse the samples
        # in the output.
        # Each sample is a dictionary with the following remarkable fields:
        #  - time: the sample time as a datetime
        #  - vals: a list of tuples, containing the data for the sample
        # In this case, since our view doesn't have keys, each sample contains
        # exactly one value. The next example shows how to deal with multiple
        # values per sample.
        for sample in output:
            print '{0}\t{1}'.format(sample["t"], sample['vals'][0][0])

        # close the view
        v.close()

        ######################################################################
        # Advanced get_data() usage.
        # get_data() has some pretty powerful functionality for server-side
        # sorting and filtering of view output data.
        # This example splits the data in 1 minute intervals, and for each
        # interval it displays the top 3 talkers.
        ######################################################################

        # Specify the column list
        columns = [
            Key(self.netshark.columns.ip.src),
            Value(self.netshark.columns.generic.bytes)
        ]

        # The list of filters is passes as a parameter to create_view()
        v = self.netshark.create_view(source, columns)

        # Retrieve the view data.
        delta = datetime.timedelta(minutes=1)
        output = v.get_data(delta=delta,            # 1 minute samples
                            sortby=1,               # Sort by column 1 (bytes)
                            sorttype='descending',  # Sort from big to small
                            fromentry=0,            # For each 1 min sample,
                                                    # the first element to
                                                    # display is the biggest
                                                    # one.
                            toentry=2               # For each 1 min sample,
                                                    # the last element to
                                                    # display is the third
                                                    # biggest one.
                            )

        # Proceed printing the data to the screen
        for sample in output:
            print sample["t"]
            for vals in sample['vals']:
                print '\t{0}\t{1}'.format(vals[0], vals[1])

        v.close()


if __name__ == '__main__':
    NetSharkApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import tempfile
import common
import testscenarios

class Dpi(common.SetUpTearDownMixin, testscenarios.TestWithScenarios):
    scenarios = common.config.get('5.0')

    def test_job_dpi(self):
        job = common.setup_capture_job(self.shark)
        columns, filters = common.setup_defaults()
        job.dpi_enabled = True
        job.save()

        job2 = self.shark.get_capture_job_by_name(job.name)
        self.assertEqual(job.dpi_enabled, True)


    def test_port_definitions(self):
        pd = self.shark.settings.port_definitions
        settings = pd.get()
        self.assertNotEqual(pd.data, None)

        try:
            pd.remove('steelscript', 65345)
        except ValueError:
            #it's all good, we don't have the rule in the server
            pass

        pd.add('steelscript', 65345, 'tcp', True)
        pd.save()

        pd.remove('steelscript', 65345)
        pd.save()

        self.assertEqual(settings, pd.get())

    def test_port_groups(self):
        gd = self.shark.settings.port_groups
        settings = gd.get()

        try:
            gd.remove('steelscript')
        except ValueError:
            #it's all good, no rule on server
            pass

        gd.add('steelscript', '1,2,3-80', priority=2)
        gd.save()

        gd.remove('steelscript')
        gd.save()

        self.assertEqual(settings, gd.get())

    def test_custom_applications(self):
        ca = self.shark.settings.custom_applications
        settings = ca.get()

        try:
            ca.remove('steelscript')
        except ValueError:
            #it's all good, no rule on server
            pass

        ca.add('steelscript', 'http://test.com')
        ca.save()

        ca.remove('steelscript')
        ca.save()

        self.assertEqual(settings, ca.get())


    def test_download_upload_port_definitions(self):
        pd = self.shark.settings.port_definitions
        settings = pd.get()
        with tempfile.NamedTemporaryFile() as f:
            pd.download(f.name)
            f.flush()
            f.seek(0)
            pd.load(f.name)
        self.assertEqual(settings, pd.get(force=True))
